(*:Package Version: 1.07 *)

(*:Version History: 1.01 (2008.07.26 - 2008.07.29)
                    1.02 (2008.07.30 - 2008.08.06)
                    1.03 (2008.08.07 - 2008.08.24)
                    1.04 (2008.08.25 - 2008.12.20)
                    1.05 (2008.12.21 - 2009.05.08)
                    1.06 (2009.05.09 - 2009.05.26)
                    1.07 (2009.05.27 -     .  .  ) *)

(*:Copyright: Copyright 2008.07-2009.05, Souitiro Ikebe *)

(*:Author: Souitiro Ikebe *)

(*:Name: Graphics`FunctionPlot` *)

(*:Title: FunctionPlot *)

(*:Keywords: 
  PoleRemovalPlot, GridLinesPlot, CommonGridLinesPlot, 
  GridLinesParametricPlot, 
  AbsColorPlot3D, AbsColorDensityPlot, ArgColorPlot3D, ArgColorDensityPlot, 
  ArgColorLogPlot3D, ArgColorArcSinhPlot3D, 
  RiemannSpherePlot3D, RiemannSphereDensityPlot, RiemannSphereLogPlot3D, 
  RiemannSphereArcSinhPlot3D, 
  ReAndImPlot3D, RePlot3D, ImPlot3D, AbsAndArgDensityPlot, 
  FundamentalRegionPlot, FundamentalRegionPlot2, DensityContourPlot, 
  RegionAbsAndArgDensityPlot, RegionAbsDensityPlot, RegionArgDensityPlot *)

(*:Sources: 

  Roman Maeder, Programming in Mathematica (Second Edition), 
  Addison-Wesley Publishing Company,Inc. (1991).

  Theodore W. Gray & Jerry Glynn, Exploring mathematics with Mathematica, 
  Addison-Wesley Publishing Company,Inc. (1991).

*)

(********************************************************************************)




BeginPackage["Graphics`FunctionPlot`",
"Graphics`ParametricPlot3D`"]




Unprotect[
  PoleRemovalPlot, GridLinesPlot, CommonGridLinesPlot, 
  GridLinesParametricPlot, GridLinesColor, FundamentalRectangle, 
  AbsColorPlot3D, AbsColorDensityPlot, ArgColorPlot3D, ArgColorDensityPlot, 
  ArgColorLogPlot3D, ArgColorArcSinhPlot3D, 
  AbsInterval, ArgInterval, IntervalColor, ComplexColorSelect, PhaseAngleColor, 
  PhaseRegionColor, ComplexRegionColor, SpecularExponent, TicksExpStep, 
  RiemannSpherePlot3D, RiemannSphereDensityPlot, RiemannSphereLogPlot3D, 
  RiemannSphereArcSinhPlot3D, 
  RiemannSphereRadius, ZeroPointAltitude, FunctionMaxRange, RiemannSphereType, 
  ReAndImPlot3D, RePlot3D, ImPlot3D, ReColorPair, ImColorPair, MaxColor, 
  ColorSeparateNum, PlotArray, Abscissa, Ordinate, 
  AbsAndArgDensityPlot, AbsoluteMaxRange, 
  FundamentalRegionPlot, FundamentalRegionPlot2, FundamentalRegionColor, 
  FundamentalRegionType, AbsContourRange, AbsContour, ArgContour, 
  AbsContourStyle, ArgContourStyle, 
  DensityContourPlot, DensityRegionColor, 
  RegionAbsAndArgDensityPlot, RegionAbsDensityPlot, RegionArgDensityPlot, 
  RegionOutsideColor]




PoleRemovalPlot::usage = 
"PoleRemovalPlot[f,{x, xmin, xmax},opts] is used to GridLinesPlot
 one real-valued functions which have poles on not draw lines."


GridLinesPlot::usage = 
"GridLinesPlot[f,{x, xmin, xmax},opts] plots the image of the two
 dimension plot of the function f with grid lines."

CommonGridLinesPlot::usage = 
"CommonGridLinesPlot[f,{x, xmin, xmax},opts] plots the image of the two
 dimension plot of the function f with grid lines.
 It's not use PoleRemovalPlot."

GridLinesParametricPlot::usage = 
"GridLinesParametricPlot[{f1, f2},{t, tmin, tmax},opts] plots the
 image of the two dimension parametric plot of the function f1 and
 f2 with grid lines."


GridLinesColor::usage = 
"GridLinesColor -> {xcolor,ycolor} is an option of GridLinesPlot and 
 GridLinesParametricPlot that gives the color of lines to draw."
 This option Default is GridLinesColor -> {Hue[0.5],Hue[0.5]}."

FundamentalRectangle::usage = 
"FundamentalRectangle -> {{x0,y0},{x1,y1}} is an option of
 GridLinesPlot & GridLinesParametricPlot that gives the fundamental 
 rectangle of grid lines paper."
 This option Default is FundamentalRectangle -> {{0,0},{1,1}}."
	


AbsColorPlot3D::usage = 
"AbsColorPlot3D[f[z],{z,z0,z1},{options}] plots f[z] in a rectangle
 in the complex plane. 
 The absolute value is represented by the height and absolute is 
 represented by the hue."

AbsColorDensityPlot::usage = 
"AbsColorDensityPlot[f[z],{z,z0,z1},{options}] plots f[z] in a
 rectangle in the complex plane. 
 The absolute is represented by the hue and phase value is 
 represented by the dark & normal coloring."

ArgColorPlot3D::usage = 
"ArgColorPlot3D[f[z],{z,z0,z1},{options}] plots f[z] in a rectangle
 in the complex plane. 
 The absolute value is represented by the height and phase is 
 represented by the hue."

ArgColorDensityPlot::usage = 
"ArgColorDensityPlot[f[z],{z,z0,z1},{options}] plots f[z] in a
 rectangle in the complex plane. 
 The phase is represented by the hue and absolute value is 
 represented by the dark & normal coloring."

ArgColorLogPlot3D::usage = 
"ArgColorLogPlot3D[f[z],{z,z0,z1},{options}] plots f[z] in a rectangle
 in the complex plane. 
 The absolute value is represented by the log(10,f[z]) scaled height 
 and phase is represented by the hue."

ArgColorArcSinhPlot3D::usage = 
"ArgColorArcSinhPlot3D[f[z],{z,z0,z1},{options}] plots f[z] in a rectangle
 in the complex plane. 
 The absolute value is represented by the arcsinh(f[z]) scaled height 
 and phase is represented by the hue."


AbsInterval::usage = 
"AbsInterval -> {functType,{cnst1,cnst2}} is an option of AbsColorPlot3D,
 AbsColorDensityPlot, ArgColorPlot3D & ArgColorDensityPlot.
 functType is functions of absolute contour style specification 
 integer number from 1 to 4.
 cnst1,cnst2 is constant use in functions of absolute contour style.
 This option Default is AbsInterval -> {1,{2,4}}."

ArgInterval::usage = 
"ArgInterval -> cnst3 is an option of AbsColorPlot3D, AbsColorDensityPlot, 
 ArgColorPlot3D & ArgColorDensityPlot.
 cnst3 is constant used in functions of phase contour style.
 This option Default is ArgInterval -> 8."

IntervalColor::usage = 
"IntervalColor -> {cnst4,cnst5} is an option of AbsColorPlot3D,
 AbsColorDensityPlot, ArgColorPlot3D & ArgColorDensityPlot.
 cnst4 is 0-1 saturation code number of the contours interval color.
 cnst5 is 0-1 brightness code number of the contours interval color.
 This option Default is IntervalColor -> {0.8,0.9}."

ComplexColorSelect::usage = 
"ComplexColorSelect -> colorType is an option of AbsColorPlot3D,
 AbsColorDensityPlot, ArgColorPlot3D & ArgColorDensityPlot.
 colorType is a PhaseAngleColor or PhaseRegionColor."

PhaseAngleColor::usage = 
"PhaseAngleColor is an optionvalue of ComplexColorSelect."

PhaseRegionColor::usage = 
"PhaseRegionColor is an optionvalue of ComplexColorSelect."

ComplexRegionColor::usage = 
"ComplexRegionColor -> {color1,color2} is an option of AbsColorPlot3D,
 AbsColorDensityPlot, ArgColorPlot3D & ArgColorDensityPlot.
 color1 is color of positive phase regions and color2 is color of 
 negative phase regions.
 This option efficient at ComplexColorSelect -> PhaseRegionColor only.
 This option Default is ComplexRegionColor -> {Hue[0],Hue[0.5]}."

SpecularExponent::usage = 
"SpecularExponent -> specNum is an option of AbsColorPlot3D,
 ArgColorPlot3D, ReAndImPlot3D, RePlot3D & ImPlot3D.
 specNum is a specular exponent of lighting on surface.
 This option Default is SpecularExponent -> 200 (AbsColorPlot3D, 
 ArgColorPlot3D), or SpecularExponent -> 30 (ReAndImPlot3D, 
 RePlot3D, ImPlot3D)."

TicksExpStep::usage = 
"TicksExpStep -> intNum is an option of ArgColorLogPlot3D & 
 ArgColorArcSinhPlot3D.
 intNum is a integer of scaling ticks step of exponent.
 This option Default is TicksExpStep -> 1."



RiemannSpherePlot3D::usage = 
"RiemannSpherePlot3D[f[z],{z,r,{phi0,phi1}},{options}] plots f[z] in a circle
 with radius r in the complex plane transform to the Riemann-sphere. 
 The absolute value is represented by the height and phase is 
 represented by the hue."

RiemannSphereDensityPlot::usage = 
"RiemannSphereDensityPlot[f[z],{z,r,{phi0,phi1}},{options}] plots f[z] in a
 circle in the complex plane transform on the Riemann-sphere. 
 The phase is represented by the hue and absolute value is 
 represented by the dark & normal coloring."

RiemannSphereLogPlot3D::usage = 
"RiemannSphereLogPlot3D[f[z],{z,r,{phi0,phi1}},{options}] plots f[z] in a circle
 in the complex plane transform to the Riemann-sphere. 
 The absolute value is represented by the log(10,f[z]) scaled height and phase is 
 represented by the hue."

RiemannSphereArcSinhPlot3D::usage = 
"RiemannSphereArcSinhPlot3D[f[z],{z,r,{phi0,phi1}},{options}] plots f[z] in a circle
 in the complex plane transform to the Riemann-sphere. 
 The absolute value is represented by the arcsinh(f[z]) scaled height and phase is 
 represented by the hue."


RiemannSphereRadius::usage = 
"RiemannSphereRadius -> r is an option of RiemannSpherePlot3D,
 RiemannSphereDensityPlot, RiemannSphereLogPlot3D & RiemannSphereArcSinhPlot3D.
 r is value of the radius of Riemann-sphere.
 This option Default is RiemannSphereRadius -> 0.5."

ZeroPointAltitude::usage = 
"ZeroPointAltitude -> val is an option of RiemannSpherePlot3D,
 RiemannSphereDensityPlot, RiemannSphereLogPlot3D & RiemannSphereArcSinhPlot3D.
 val is value of the height of zero point on the Riemann-sphere.
 This option Default is ZeroPointAltitude -> 1."

FunctionMaxRange::usage = 
"FunctionMaxRange -> val is an option of RiemannSpherePlot3D,
 RiemannSphereDensityPlot, RiemannSphereLogPlot3D & RiemannSphereArcSinhPlot3D.
 val is value of the height of maximum range on the Riemann-sphere.
 This option Default is FunctionMaxRange -> 9."

RiemannSphereType::usage = 
"RiemannSphereType -> cnst is an option of RiemannSpherePlot3D,
 RiemannSphereDensityPlot, RiemannSphereLogPlot3D & RiemannSphereArcSinhPlot3D.
 cnst=1 are upper is infinity type, and cnst=2 are lower is infinity type 
 of the Riemann-sphere.
 This option Default is RiemannSphereType -> 1."



ReAndImPlot3D::usage = 
"ReAndImPlot3D[f[z],{z,z0,z1},{options}] plots f[z] in a rectangle
 in the complex plane. 
 The absolute value is represented by the real part and imaginal part."

RePlot3D::usage = 
"ReAndImPlot3D[f[z],{z,z0,z1},{options}] plots f[z] in a rectangle
 in the complex plane. 
 The absolute value is represented by the real part only."

ImPlot3D::usage = 
"ReAndImPlot3D[f[z],{z,z0,z1},{options}] plots f[z] in a rectangle
 in the complex plane. 
 The absolute value is represented by the imaginal part only."


ReColorPair::usage = 
"ReColorPair -> {ncolor,pcolor} is an option of ReAndImPlot3D & RePlot3D.
 ncolor is color of negative real function values and pcolor is 
 color of positive real function values.
 This option Default is ReColorPair -> {Hue[0.5],Hue[0]}."

ImColorPair::usage = 
"ImColorPair -> {ncolor,pcolor} is an option of ReAndImPlot3D & ImPlot3D.
 ncolor is color of negative imaginal function values and pcolor is 
 color of positive imaginal function values.
 This option Default is ImColorPair -> {Hue[0.75],Hue[0.25]}."

MaxColor::usage = 
"MaxColor -> {{reNcolor,rePcolor},{imNcolor,imPcolor}} is an option of
 ReAndImPlot3D.
 reNcolor & rePcolor is color of negative & positive real function 
 values of maximum.
 imNcolor & imPcolor is color of negative & positive imaginal function 
 values of maximum.
 This option Default is MaxColor -> {{GrayLevel[0.3],GrayLevel[0.3]},
 {GrayLevel[0.3],GrayLevel[0.3]}}.
 MaxColor -> {ncolor,pcolor} is an option of RePlot3D & ImPlot3D.
 ncolor is color of negative real or imaginal function values of maximum.
 pcolor is color of positive real or imaginal function values of maximum.
 This option Default is MaxColor -> {GrayLevel[0.3],GrayLevel[0.3]}."

ColorSeparateNum::usage = 
"ColorSeparateNum -> num is an option of ReAndImPlot3D, RePlot3D & ImPlot3D.
 num is integer number of step of MaxColor.
 This option Default is ColorSeparateNum -> 8."

PlotArray::usage = 
"PlotArray -> arraytype is an option of ReAndImPlot3D,
 AbsAndArgDensityPlot & RegionAbsAndArgDensityPlot.
 arraytype is an Abscissa or Ordinate.
 This option Default is PlotArray -> Abscissa."

Abscissa::usage = 
"Abscissa is an optionvalue of PlotArray."

Ordinate::usage = 
"Ordinate is an optionvalue of PlotArray."



AbsAndArgDensityPlot::usage = 
"AbsAndArgDensityPlot[f[z],{z,z0,z1},{options}] plots f[z] in a
 rectangle in the complex plane. 
 The absolute value is represented by the absolute value and phase value."


AbsoluteMaxRange::usage = 
"AbsoluteMaxRange -> max is an option of AbsAndArgDensityPlot,
 RegionAbsAndArgDensityPlot & RegionAbsDensityPlot.
 This option Default is AbsoluteMaxRange -> 10^8."



FundamentalRegionPlot::usage = 
"FundamentalRegionPlot[f[z],{z,z0,z1},{options}] plots f[z] in a
 rectangle in the complex plane. 
 The absolute value is represented by the absolute value and phase value."

FundamentalRegionPlot2::usage = 
"FundamentalRegionPlot2[f[z],{z,z0,z1},{options}] plots f[z] in a
 rectangle in the complex plane. 
 The absolute value is represented by the absolute value and phase value."


FundamentalRegionColor::usage = 
"FundamentalRegionColor -> {color1,color2} is an option of
 FundamentalRegionPlot.
 color1 is color of positive phase regions and color2 is color of 
 negative phase regions.
 This option Default is FundamentalRegionColor -> {Hue[0],Hue[0.5]}."

FundamentalRegionType::usage = 
"FundamentalRegionType -> type is an option of FundamentalRegionPlot.
 type 1 is uni color or type 2 is gradation color on region.
 This option Default is FundamentalRegionType -> 2."

AbsContourRange::usage = 
"AbsContourRange -> {cmin,cmax} is an option of FundamentalRegionPlot
 & DensityContourPlot.
 This option Default is AbsContourRange -> {10^-30, 10^30}."

AbsContour::usage = 
"AbsContour -> {functType,{cnst1,cnst2}} is an option of
 FundamentalRegionPlot & DensityContourPlot.
 functType is functions of absolute contours style specification 
 integer number from 1 to 4.
 cnst1,cnst2 is constant use in functions of absolute contours style.
 This option Default is AbsContour -> {1,{2,4}}."

ArgContour::usage = 
"ArgContour -> kunit is an option of FundamentalRegionPlot &
 DensityContourPlot.
 Contours plots by Join[Table[k Pi/kunit,{k,-kunit+1,-1}],
 Table[k Pi/kunit,{k,1,kunit-1}]].
 This option Default is ArgContour -> 8."

AbsContourStyle::usage = 
"AbsContourStyle -> style1 or AbsContourStyle -> {{style1,...}} is
 an option of FundamentalRegionPlot & DensityContourPlot.
 style1 or {{style1,...}} is style of absolute value contour lines.
 This option Default is AbsContourStyle -> {{GrayLevel[0],Thickness[0.0005]}} 
 at FundamentalRegionPlot 
 or AbsContourStyle -> {{GrayLevel[0],Thickness[0.0005]}} 
 at DensityContourPlot."

ArgContourStyle::usage = 
"ArgContourStyle -> style1 or ArgContourStyle -> {{style1,...}} is
 an option of FundamentalRegionPlot & DensityContourPlot.
 style1 or {{style1,...}} is style of phase value contour lines.
 This option Default is ArgContourStyle -> {{Hue[0.7],Thickness[0.0005]}} 
 at FundamentalRegionPlot 
 or ArgContourStyle -> {{AbsoluteDashing[{3,3}],GrayLevel[0],Thickness[0.0005]}} 
 at DensityContourPlot."



DensityContourPlot::usage = 
"DensityContourPlot[f[z],{z,z0,z1},{options}] plots f[z] in a
 rectangle in the complex plane. 
 The absolute value is represented by the absolute value and phase value."


DensityRegionColor::usage = 
"DensityRegionColor -> n is an option of DensityContourPlot.
 n is an integer nember and infinity colors of phase value.
 This option Default is DensityRegionColor -> 8."



RegionAbsAndArgDensityPlot::usage = 
"RegionAbsAndArgDensityPlot[f[z],r[z],{z,z0,z1},{options}] plots
 f[z] in a rectangle in the complex plane. 
 The absolute value is represented by the absolute value and phase value."

RegionAbsDensityPlot::usage = 
"RegionAbsDensityPlot[f[z],r[z],{z,z0,z1},{options}] plots f[z] in
 a rectangle in the complex plane. 
 The absolute value is represented by the absolute value only."

RegionArgDensityPlot::usage = 
"RegionArgDensityPlot[f[z],r[z],{z,z0,z1},{options}] plots f[z] in
 a rectangle in the complex plane. 
 The absolute value is represented by the phase value only."


RegionOutsideColor::usage = 
"RegionOutsideColor -> {color1,color2} is an option of
 RegionAbsAndArgDensityPlot, RegionAbsDensityPlot & 
 RegionArgDensityPlot.
 color1 is 0-1 saturation code number and color2 is 0-1 brightness 
 code number.
 This option Default is RegionOutsideColor -> {1,0.6}."





Begin["`Private`"]

Needs["Utilities`FilterOptions`"]





(******************** Common Code ********************)

zrAbs[z_]:=If[NumericQ[z]=!=True,0,Abs[z]];

zrArg[z_]:=If[NumericQ[z]=!=True,0,If[Abs[z]>0,Arg[z],0]];

absOfArg[z_]:=If[NumericQ[z]=!=True,0,If[Abs[z]>0,Abs[Arg[z]],0]];


abs3DMax[f_,ma_]:=
  Module[{absfun},absfun=Abs[f];If[absfun>=ma,ma*0.99999999999,absfun]];

abs3DLogMax[f_,{mi_,ma_}]:=
  Module[{absfun},absfun=Log[10,Abs[f]];
    If[absfun<mi,mi,If[absfun>ma,ma,absfun]]];

abs3DArcSinhMax[f_,ma_]:=
  Module[{absfun},absfun=ArcSinh[Abs[f]];If[absfun>ma,ma,absfun]];


changeHue[h_,cSamp_]:=
  If[Head[cSamp]===GrayLevel,
    Hue[h,0,cSamp[[1]]],
  If[Head[cSamp]===Hue && Length[cSamp]===3 && cSamp[[2]]===0,
    Hue[h,0,cSamp[[3]]],
  If[Head[cSamp]===Hue && Length[cSamp]===1,
    Hue[cSamp[[1]],1,1],cSamp]]];


sawWave[z_]:=z-Floor[z];


ccSHue[h_,u_,e_,ccSel_,{color1_,color2_}]:=
  If[ccSel===1,Hue[h,u,e],
    If[Mod[Floor[2*Re[h]],2]===0,Hue[h/5-Floor[2*h]/10+color1[[1]],u,e],
       Hue[h/5-Floor[2*h]/10+color2[[1]],u,e]]];


fontSizeFunct[x_]:=
  If[Abs[N[x]]===1.,1,
    If[Abs[N[x]]<1,
      0.901061849843*Sqrt[x]+0.116101610661*x,
      -0.018197245638+0.497677504024*Sqrt[1/x]+0.547671099744/x]];





(******************** GridLinesPlot ********************)

Options[GridLinesPlot]=
  Join[{GridLinesColor->Automatic, 
        FundamentalRectangle->Automatic},Options[Plot]];


GridLinesPlot[unknownFunc_,{x_,xmin_,xmax_},opts___]:=
  Module[{plotrange,background,gridlinescolor,fundamentalrectangle},
    plotrange=PlotRange/.{opts}/.Options[Plot];
    background=Background/.{opts}/.Options[Plot];
    gridlinescolor=GridLinesColor/.{opts}/.Options[GridLinesPlot];
    fundamentalrectangle=FundamentalRectangle/.{opts}/.Options[GridLinesPlot];
     If[(background===Automatic || background===None),background=GrayLevel[1]];
     If[gridlinescolor===Automatic,gridlinescolor={Hue[0.5],Hue[0.5]}];
     If[fundamentalrectangle===Automatic,fundamentalrectangle={{0,0},{1,1}}];
   PoleRemovalPlot[unknownFunc,{x,xmin,xmax},
     GridLines->{gridltable[{fundamentalrectangle[[1,1]],
      fundamentalrectangle[[2,1]]},{xmin,xmax},
      {gridlinescolor[[1]],background}],
      gridltable[{fundamentalrectangle[[1,2]],fundamentalrectangle[[2,2]]},
      {plotrange[[1]],plotrange[[2]]},
      {gridlinescolor[[2]],background}]},
   Evaluate[FilterOptions[Plot,opts]]]]


gridltable[{x0_,x1_},{xmin_,xmax_},{gridCol_,backCol_}]:=
  Module[{dx=(x1-x0)},
  Table[{x0+i dx,{blendColor[gridCol,backCol,blendNum[i]]}},
     {i,IntegerPart[(xmin-x0)/dx-1],IntegerPart[(xmax-x0)/dx+1],1/10}]]/;
  (MemberQ[{Hue,GrayLevel},Head[gridCol]] && MemberQ[{Hue,GrayLevel},Head[backCol]])

blendNum[i_]:=
  If[Mod[10 i,10]===0,0,If[Mod[10 i,10]===5,1,2]];


blendColor[cInit_,cBlen_,m_]:=
  Module[{ci,cb},
    ci=changeHue[cBlen[[1]],cInit];
    cb=changeHue[cInit[[1]],cBlen];
   Nest[meanHue[#,cb]&,changeHue[cb[[1]],ci],m]]

meanHue[c1_,c2_]:=
  Apply[Hue,Map[((c1[[#]]+c2[[#]])/2)&,{1,2,3}]];





(******************** PoleRemovalPlot ********************)

Options[PoleRemovalPlot]=Options[Plot]


PoleRemovalPlot[f_List,{x_,x0_,x1_},options___Rule]:=
  Module[{graphicsList,plotstyle},
      plotstyle=PlotStyle/.{options}/.Options[Plot];
    graphicsList=MapThread[
      PoleRemovalPlot[#1,{x,x0,x1},PlotStyle->#2,options,
        DisplayFunction->Identity]&,{f,plotstyle}];
    Show[graphicsList,DisplayFunction->$DisplayFunction]]


PoleRemovalPlot[f_,{x_,x0_,x1_},options___Rule]:=
  Module[{firstplot,grapRatio,minmax,minimum,maximum,pointList,
          coordList,plotCoords,numberOfPoints,graphicsOptions,
          displayFunction,i,threshold,plotstyle},
      displayFunction=DisplayFunction/.{options}/.Options[Plot];
      plotstyle=PlotStyle/.{options}/.Options[Plot];
        If[plotstyle===Automatic,plotstyle=GrayLevel[0]];
    firstplot=Plot[f,{x,x0,x1},DisplayFunction->Identity,PlotPoints->1000,
        Evaluate[FilterOptions[Graphics,options]]];
      pointList=Flatten[Apply[List,firstplot[[1,1]],2],2];
      graphicsOptions=Apply[Sequence,firstplot[[2]]];
      grapRatio=FullOptions[firstplot,AspectRatio];
      minmax=FullOptions[firstplot,PlotRange][[2]];
      {minimum,maximum}={minmax[[1]],minmax[[2]]};
      numberOfPoints=Length[pointList];
      plotCoords={};
      coordList={pointList[[1]]};
    Do[If[((pointList[[i-1]][[2]]<minimum && pointList[[i]][[2]]>maximum)||
           (pointList[[i-1]][[2]]>maximum && pointList[[i]][[2]]<minimum)),
         AppendTo[plotCoords,coordList];coordList={},
         AppendTo[coordList,pointList[[i]]]],
       {i,2,numberOfPoints-1}];
    AppendTo[coordList,pointList[[numberOfPoints]]];
    AppendTo[plotCoords,coordList];
    Show[Graphics[
      Table[Join[Flatten[{plotstyle}],{Line[plotCoords[[j]]]}],
        {j,1,Length[plotCoords]}]],
      DefaultFont->{"Courier",10*fontSizeFunct[grapRatio]},
      DisplayFunction->displayFunction,graphicsOptions]]/;(Head[f]=!=List)





(******************** CommonGridLinesPlot ********************)

Options[CommonGridLinesPlot]=
  Join[{GridLinesColor->Automatic, 
        FundamentalRectangle->Automatic},Options[Plot]];


CommonGridLinesPlot[unknownFunc_,{x_,xmin_,xmax_},opts___]:=
  Module[{plotrange,background,gridlinescolor,fundamentalrectangle},
    plotrange=PlotRange/.{opts}/.Options[Plot];
    background=Background/.{opts}/.Options[Plot];
    gridlinescolor=GridLinesColor/.{opts}/.Options[CommonGridLinesPlot];
    fundamentalrectangle=FundamentalRectangle/.{opts}/.Options[CommonGridLinesPlot];
     If[(background===Automatic || background===None),background=GrayLevel[1]];
     If[gridlinescolor===Automatic,gridlinescolor={Hue[0.5],Hue[0.5]}];
     If[fundamentalrectangle===Automatic,fundamentalrectangle={{0,0},{1,1}}];
   Plot[unknownFunc,{x,xmin,xmax},
     GridLines->{gridltable[{fundamentalrectangle[[1,1]],
      fundamentalrectangle[[2,1]]},{xmin,xmax},
      {gridlinescolor[[1]],background}],
      gridltable[{fundamentalrectangle[[1,2]],fundamentalrectangle[[2,2]]},
      {plotrange[[1]],plotrange[[2]]},
      {gridlinescolor[[2]],background}]},
   Evaluate[FilterOptions[Plot,opts]]]]





(******************** GridLinesParametricPlot ********************)

Options[GridLinesParametricPlot]=
  Join[{GridLinesColor->Automatic, 
        FundamentalRectangle->Automatic},Options[ParametricPlot]];


GridLinesParametricPlot[{unknownFunc1st_,unknownFunc2nd_},{t_,tmin_,tmax_},opts___]:=
  Module[{paraPl,grapRatio,plotrange,background,gridlinescolor,fundamentalrectangle},
    plotrange=PlotRange/.{opts}/.Options[ParametricPlot];
    background=Background/.{opts}/.Options[ParametricPlot];
    gridlinescolor=GridLinesColor/.{opts}/.Options[GridLinesPlot];
    fundamentalrectangle=FundamentalRectangle/.{opts}/.Options[GridLinesPlot];
     If[(background===Automatic || background===None),background=GrayLevel[1]];
     If[gridlinescolor===Automatic,gridlinescolor={Hue[0.5],Hue[0.5]}];
     If[fundamentalrectangle===Automatic,fundamentalrectangle={{0,0},{1,1}}];
   paraPl=ParametricPlot[{unknownFunc1st,unknownFunc2nd},{t,tmin,tmax},
     GridLines->{gridltable[{fundamentalrectangle[[1,1]],fundamentalrectangle[[2,1]]},
      {plotrange[[1,1]],plotrange[[1,2]]},
      {gridlinescolor[[1]],background}],
      gridltable[{fundamentalrectangle[[1,2]],fundamentalrectangle[[2,2]]},
      {plotrange[[2,1]],plotrange[[2,2]]},
      {gridlinescolor[[2]],background}]},
     DisplayFunction->Identity,Evaluate[FilterOptions[ParametricPlot,opts]]];
   grapRatio=FullOptions[paraPl,AspectRatio];
   Show[paraPl,DefaultFont->{"Courier",10*fontSizeFunct[grapRatio]},
      DisplayFunction->$DisplayFunction]]
        




(******************** AbsColorPlot3D ********************)

Options[AbsColorPlot3D]=
    Join[{AbsInterval->Automatic,
          ArgInterval->Automatic,
          IntervalColor->Automatic,
          ComplexColorSelect->Automatic,
          ComplexRegionColor->Automatic,
          SpecularExponent->Automatic},Options[Plot3D]];


AbsColorPlot3D[unknownFunc_,{z_,z0_,z1_},opts___]:=
  Module[{ukfval,x,y,absP3,grapRatio,plotrange,absinterval,arginterval,
          intervalcolor,complexcolorselect,complexregioncolor,specularexp},
    plotrange=PlotRange/.{opts}/.Options[AbsColorPlot3D];
    absinterval=AbsInterval/.{opts}/.Options[AbsColorPlot3D];
    arginterval=ArgInterval/.{opts}/.Options[AbsColorPlot3D];
    intervalcolor=IntervalColor/.{opts}/.Options[AbsColorPlot3D];
    complexcolorselect=ComplexColorSelect/.{opts}/.Options[AbsColorPlot3D];
    complexregioncolor=ComplexRegionColor/.{opts}/.Options[AbsColorPlot3D];
    specularexp=SpecularExponent/.{opts}/.Options[AbsColorPlot3D];
     If[absinterval===Automatic,absinterval={1,{2,4}}];
     If[arginterval===Automatic,arginterval=8];
     If[intervalcolor===Automatic,intervalcolor={0.8,0.9}];
     If[(complexcolorselect===Automatic || complexcolorselect===PhaseAngleColor),
         complexcolorselect=1,complexcolorselect=2];
     If[complexregioncolor===Automatic,complexregioncolor={Hue[0],Hue[0.5]}];
     If[specularexp===Automatic,specularexp=200];
   absP3=Plot3D[{abs3DMax[ukfval=unknownFunc/.z->(x+I y),plotrange[[2]]],
     absColorF[ukfval,complexcolorselect,
       {absinterval[[1]],{absinterval[[2,1]],absinterval[[2,2]],arginterval}},
       {intervalcolor[[1]],intervalcolor[[2]]},
       {complexregioncolor[[1]],complexregioncolor[[2]]}]},
     {x,Re[z0],Re[z1]},{y,Im[z0],Im[z1]},
     DisplayFunction->Identity,Evaluate[FilterOptions[Plot3D,opts]]];
   grapRatio=FullOptions[absP3,AspectRatio];
   Show[Graphics3D[absP3]/.
     Hue[h_,s_,b_]->SurfaceColor[Hue[h,s,b],RGBColor[1,1,1],specularexp],
     Lighting->True,LightSources->{{{1,0,1},Hue[0,0.5,0.6]},
       {{1,1,1},Hue[1/3,0.5,0.6]},{{0,1,1},Hue[2/3,0.5,0.6]}},
     DefaultFont->{"Courier",10*fontSizeFunct[grapRatio]},
     DisplayFunction->$DisplayFunction]]


absColorF[z_,ccSel_,{intFType_,{logIntV_,absIntV_,argIntV_}},{iSatu_,iBrig_},{c1_,c2_}]:=
  If[NumericQ[z]===False,Hue[0,0,0],
  If[N[zrAbs[z]]===0.,Hue[0,0,1],
  If[Mod[Floor[absIntervalFunc[intFType,logIntV,z]*absIntV],2]===1 && 
     Mod[Floor[zrArg[z]*argIntV*2/Pi],2]===1,
       ccSHue[sawWave[absIntervalFunc[intFType,10*logIntV,z]],
         iSatu,iBrig,ccSel,{c1,c2}],
  If[Mod[Floor[absIntervalFunc[intFType,logIntV,z]*absIntV],2]===1 && 
     Mod[Floor[zrArg[z]*argIntV*2/Pi],2]===0,
       ccSHue[sawWave[absIntervalFunc[intFType,10*logIntV,z]],
         iSatu,1,ccSel,{c1,c2}],
  If[Mod[Floor[absIntervalFunc[intFType,logIntV,z]*absIntV],2]===0 && 
     Mod[Floor[zrArg[z]*argIntV*2/Pi],2]===1,
       ccSHue[sawWave[absIntervalFunc[intFType,10*logIntV,z]],
         1,iBrig,ccSel,{c1,c2}],
       ccSHue[sawWave[absIntervalFunc[intFType,10*logIntV,z]],
         1,1,ccSel,{c1,c2}]]]]]];


absIntervalFunc[t_,a_,z_]:=
  If[t===1,Log[a,zrAbs[z]],
  If[t===2,If[zrAbs[z]<1,Log[a,zrAbs[z]],(zrAbs[z]-1)/Log[a]],
  If[t===3,Log[a,Log[a,(a-1)*zrAbs[z]+1]],
  If[t===4,Log[a,Log[a,Log[a,(a^a-a)*zrAbs[z]+a]]],
  If[t===5,
    If[zrAbs[z]<1,-Log[a,Log[a,(a-1)/zrAbs[z]+1]],
           Log[a,Log[a,(a-1)*zrAbs[z]+1]]],
    If[zrAbs[z]<1,-Log[a,Log[a,Log[a,(a^a-a)/zrAbs[z]+a]]],
           Log[a,Log[a,Log[a,(a^a-a)*zrAbs[z]+a]]]]]]]]];





(******************** AbsColorDensityPlot ********************)

Options[AbsColorDensityPlot]=
    Join[{AbsInterval->Automatic,
          ArgInterval->Automatic,
          IntervalColor->Automatic,
          ComplexColorSelect->Automatic,
          ComplexRegionColor->Automatic},Options[DensityPlot]];


AbsColorDensityPlot[unknownFunc_,{z_,z0_,z1_},opts___]:=
  Module[{x,y,absDen,grapRatio,absinterval,arginterval,intervalcolor,
          complexcolorselect,complexregioncolor},
    absinterval=AbsInterval/.{opts}/.Options[AbsColorDensityPlot];
    arginterval=ArgInterval/.{opts}/.Options[AbsColorDensityPlot];
    intervalcolor=IntervalColor/.{opts}/.Options[AbsColorDensityPlot];
    complexcolorselect=ComplexColorSelect/.{opts}/.Options[AbsColorDensityPlot];
    complexregioncolor=ComplexRegionColor/.{opts}/.Options[AbsColorDensityPlot];
      If[absinterval===Automatic,absinterval={1,{2,4}}];
      If[arginterval===Automatic,arginterval=8];
      If[intervalcolor===Automatic,intervalcolor={0.8,0.9}];
      If[(complexcolorselect===Automatic || complexcolorselect===PhaseAngleColor),
        complexcolorselect=1,complexcolorselect=2];
      If[complexregioncolor===Automatic,complexregioncolor={Hue[0],Hue[0.5]}];
    absDen=DensityPlot[absColorFuncValue[unknownFunc/.z->(x+I y),
      {absinterval[[1]],{absinterval[[2,1]],absinterval[[2,2]],arginterval}}],
      {x,Re[z0],Re[z1]},{y,Im[z0],Im[z1]},PlotRange->{0,4},
      ColorFunction->(absDensityColorF[#,complexcolorselect,
      {intervalcolor[[1]],intervalcolor[[2]]},
      {complexregioncolor[[1]],complexregioncolor[[2]]}]&),
      DisplayFunction->Identity,Evaluate[FilterOptions[DensityPlot,opts]]];
    grapRatio=FullOptions[absDen,AspectRatio];
    Show[absDen,DefaultFont->{"Courier",10*fontSizeFunct[grapRatio]},
      DisplayFunction->$DisplayFunction]]


absColorFuncValue[z_,{intFType_,{logIntV_,absIntV_,argIntV_}}]:=
  If[Mod[Floor[absIntervalFunc[intFType,logIntV,z]*absIntV],2]===1 && 
     Mod[Floor[Arg[z]*argIntV*2/Pi],2]===1,
       sawWave[absIntervalFunc[intFType,10*logIntV,z]],
  If[Mod[Floor[absIntervalFunc[intFType,logIntV,z]*absIntV],2]===1 && 
     Mod[Floor[Arg[z]*argIntV*2/Pi],2]===0,
       sawWave[absIntervalFunc[intFType,10*logIntV,z]]+1,
  If[Mod[Floor[absIntervalFunc[intFType,logIntV,z]*absIntV],2]===0 && 
     Mod[Floor[Arg[z]*argIntV*2/Pi],2]===1,
       sawWave[absIntervalFunc[intFType,10*logIntV,z]]+2,
       sawWave[absIntervalFunc[intFType,10*logIntV,z]]+3]]];

absDensityColorF[h_,ccSel_,{iSatu_,iBrig_},{c1_,c2_}]:=
  If[h<=1/4,ccSHue[4 h,iSatu,iBrig,ccSel,{c1,c2}],
  If[1/4<h<=2/4,ccSHue[4 h,iSatu,1,ccSel,{c1,c2}],
  If[2/4<h<=3/4,ccSHue[4 h,1,iBrig,ccSel,{c1,c2}],
                ccSHue[4 h,1,1,ccSel,{c1,c2}]]]];





(******************** ArgColorPlot3D ********************)

Options[ArgColorPlot3D]=
    Join[{AbsInterval->Automatic,
          ArgInterval->Automatic,
          IntervalColor->Automatic,
          ComplexColorSelect->Automatic,
          ComplexRegionColor->Automatic,
          SpecularExponent->Automatic},Options[Plot3D]];


ArgColorPlot3D[unknownFunc_,{z_,z0_,z1_},opts___]:=
  Module[{ukfval,x,y,argP3,grapRatio,plotrange,absinterval,arginterval,
          intervalcolor,complexcolorselect,complexregioncolor,specularexp},
    plotrange=PlotRange/.{opts}/.Options[ArgColorPlot3D];
    absinterval=AbsInterval/.{opts}/.Options[ArgColorPlot3D];
    arginterval=ArgInterval/.{opts}/.Options[ArgColorPlot3D];
    intervalcolor=IntervalColor/.{opts}/.Options[ArgColorPlot3D];
    complexcolorselect=ComplexColorSelect/.{opts}/.Options[ArgColorPlot3D];
    complexregioncolor=ComplexRegionColor/.{opts}/.Options[ArgColorPlot3D];
    specularexp=SpecularExponent/.{opts}/.Options[ArgColorPlot3D];
     If[absinterval===Automatic,absinterval={1,{2,4}}];
     If[arginterval===Automatic,arginterval=8];
     If[intervalcolor===Automatic,intervalcolor={0.8,0.9}];
     If[(complexcolorselect===Automatic || complexcolorselect===PhaseAngleColor),
         complexcolorselect=1,complexcolorselect=2];
     If[complexregioncolor===Automatic,complexregioncolor={Hue[0],Hue[0.5]}];
     If[specularexp===Automatic,specularexp=200];
   argP3=Plot3D[{abs3DMax[ukfval=unknownFunc/.z->(x+I y),plotrange[[2]]],
     argColorF[ukfval,complexcolorselect,
       {absinterval[[1]],{absinterval[[2,1]],absinterval[[2,2]],arginterval}},
       {intervalcolor[[1]],intervalcolor[[2]]},
       {complexregioncolor[[1]],complexregioncolor[[2]]}]},
     {x,Re[z0],Re[z1]},{y,Im[z0],Im[z1]},
     DisplayFunction->Identity,Evaluate[FilterOptions[Plot3D,opts]]];
   grapRatio=FullOptions[argP3,AspectRatio];
   Show[Graphics3D[argP3]/.
     Hue[h_,s_,b_]->SurfaceColor[Hue[h,s,b],RGBColor[1,1,1],specularexp],
     Lighting->True,LightSources->{{{1,0,1},Hue[0,0.5,0.6]},
       {{1,1,1},Hue[1/3,0.5,0.6]},{{0,1,1},Hue[2/3,0.5,0.6]}},
     DefaultFont->{"Courier",10*fontSizeFunct[grapRatio]},
     DisplayFunction->$DisplayFunction]]


argColorF[z_,ccSel_,{intFType_,{logIntV_,absIntV_,argIntV_}},{iSatu_,iBrig_},{c1_,c2_}]:=
  If[NumericQ[z]===False,Hue[0,0,0],
  If[N[zrAbs[z]]===0.,Hue[0,0,1],
  If[Mod[Floor[absIntervalFunc[intFType,logIntV,z]*absIntV],2]===1 && 
     Mod[Floor[zrArg[z]*argIntV*2/Pi],2]===1,
       ccSHue[Re[zrArg[z]/(2Pi)],iSatu,iBrig,ccSel,{c1,c2}],
  If[Mod[Floor[absIntervalFunc[intFType,logIntV,z]*absIntV],2]===1 && 
     Mod[Floor[zrArg[z]*argIntV*2/Pi],2]===0,
       ccSHue[Re[zrArg[z]/(2Pi)],1,iBrig,ccSel,{c1,c2}],
  If[Mod[Floor[absIntervalFunc[intFType,logIntV,z]*absIntV],2]===0 && 
     Mod[Floor[zrArg[z]*argIntV*2/Pi],2]===1,
       ccSHue[Re[zrArg[z]/(2Pi)],iSatu,1,ccSel,{c1,c2}],
       ccSHue[Re[zrArg[z]/(2Pi)],1,1,ccSel,{c1,c2}]]]]]];





(******************** ArgColorDensityPlot ********************)

Options[ArgColorDensityPlot]=
    Join[{AbsInterval->Automatic,
          ArgInterval->Automatic,
          IntervalColor->Automatic,
          ComplexColorSelect->Automatic,
          ComplexRegionColor->Automatic},Options[DensityPlot]];


ArgColorDensityPlot[unknownFunc_,{z_,z0_,z1_},opts___]:=
  Module[{x,y,argDen,grapRatio,absinterval,arginterval,intervalcolor,
          complexcolorselect,complexregioncolor},
    absinterval=AbsInterval/.{opts}/.Options[ArgColorDensityPlot];
    arginterval=ArgInterval/.{opts}/.Options[ArgColorDensityPlot];
    intervalcolor=IntervalColor/.{opts}/.Options[ArgColorDensityPlot];
    complexcolorselect=ComplexColorSelect/.{opts}/.Options[ArgColorDensityPlot];
    complexregioncolor=ComplexRegionColor/.{opts}/.Options[ArgColorDensityPlot];
      If[absinterval===Automatic,absinterval={1,{2,4}}];
      If[arginterval===Automatic,arginterval=8];
      If[intervalcolor===Automatic,intervalcolor={0.8,0.9}];
      If[(complexcolorselect===Automatic || complexcolorselect===PhaseAngleColor),
        complexcolorselect=1,complexcolorselect=2];
      If[complexregioncolor===Automatic,complexregioncolor={Hue[0],Hue[0.5]}];
    argDen=DensityPlot[argColorFuncValue[unknownFunc/.z->(x+I y),
      {absinterval[[1]],{absinterval[[2,1]],absinterval[[2,2]],arginterval}}],
      {x,Re[z0],Re[z1]},{y,Im[z0],Im[z1]},PlotRange->{0,8 Pi},
      ColorFunction->(argDensityColorF[#,complexcolorselect,
      {intervalcolor[[1]],intervalcolor[[2]]},
      {complexregioncolor[[1]],complexregioncolor[[2]]}]&),
      DisplayFunction->Identity,Evaluate[FilterOptions[DensityPlot,opts]]];
    grapRatio=FullOptions[argDen,AspectRatio];
    Show[argDen,DefaultFont->{"Courier",10*fontSizeFunct[grapRatio]},
      DisplayFunction->$DisplayFunction]]


argColorFuncValue[z_,{intFType_,{logIntV_,absIntV_,argIntV_}}]:=
  If[z===0,0,
  If[Mod[Floor[absIntervalFunc[intFType,logIntV,z]*absIntV],2]===1 && 
     Mod[Floor[zrArg[z]*argIntV*2/Pi],2]===1,
       zrArg[z]+Pi,
  If[Mod[Floor[absIntervalFunc[intFType,logIntV,z]*absIntV],2]===1 && 
     Mod[Floor[zrArg[z]*argIntV*2/Pi],2]===0,
       zrArg[z]+3 Pi,
  If[Mod[Floor[absIntervalFunc[intFType,logIntV,z]*absIntV],2]===0 && 
     Mod[Floor[zrArg[z]*argIntV*2/Pi],2]===1,
       zrArg[z]+5 Pi,
       zrArg[z]+7 Pi]]]];


argDensityColorF[h_,ccSel_,{iSatu_,iBrig_},{c1_,c2_}]:=
  If[0<h<=1/4,ccSHue[4 h+0.5,iSatu,iBrig,ccSel,{c1,c2}],
  If[1/4<h<=2/4,ccSHue[4 h+0.5,1,iBrig,ccSel,{c1,c2}],
  If[2/4<h<=3/4,ccSHue[4 h+0.5,iSatu,1,ccSel,{c1,c2}],
  If[3/4<h<=1,ccSHue[4 h+0.5,1,1,ccSel,{c1,c2}],
              Hue[0,0,1]]]]];





(******************** ArgColorLogPlot3D ********************)

Options[ArgColorLogPlot3D]=
    Join[{AbsInterval->Automatic,
          ArgInterval->Automatic,
          IntervalColor->Automatic,
          ComplexColorSelect->Automatic,
          ComplexRegionColor->Automatic,
          SpecularExponent->Automatic,
          TicksExpStep->Automatic},Options[Plot3D]];


ArgColorLogPlot3D[unknownFunc_,{z_,z0_,z1_},opts___]:=
  Module[{ukfval,x,y,argP3,grapRatio,plotrange,absinterval,arginterval,
          intervalcolor,complexcolorselect,complexregioncolor,specularexp,ticksexpstep},
    plotrange=PlotRange/.{opts}/.Options[ArgColorLogPlot3D];
    absinterval=AbsInterval/.{opts}/.Options[ArgColorLogPlot3D];
    arginterval=ArgInterval/.{opts}/.Options[ArgColorLogPlot3D];
    intervalcolor=IntervalColor/.{opts}/.Options[ArgColorLogPlot3D];
    complexcolorselect=ComplexColorSelect/.{opts}/.Options[ArgColorLogPlot3D];
    complexregioncolor=ComplexRegionColor/.{opts}/.Options[ArgColorLogPlot3D];
    specularexp=SpecularExponent/.{opts}/.Options[ArgColorLogPlot3D];
    ticksexpstep=TicksExpStep/.{opts}/.Options[ArgColorLogPlot3D];
     If[absinterval===Automatic,absinterval={1,{2,4}}];
     If[arginterval===Automatic,arginterval=8];
     If[intervalcolor===Automatic,intervalcolor={0.8,0.9}];
     If[(complexcolorselect===Automatic || complexcolorselect===PhaseAngleColor),
         complexcolorselect=1,complexcolorselect=2];
     If[complexregioncolor===Automatic,complexregioncolor={Hue[0],Hue[0.5]}];
     If[specularexp===Automatic,specularexp=200];
     If[ticksexpstep===Automatic,ticksexpstep=1];
   argP3=Plot3D[{Log[10,abs3DMax[ukfval=unknownFunc/.z->(x+I y),plotrange[[2]]]],
     argColorF[ukfval,complexcolorselect,
       {absinterval[[1]],{absinterval[[2,1]],absinterval[[2,2]],arginterval}},
       {intervalcolor[[1]],intervalcolor[[2]]},
       {complexregioncolor[[1]],complexregioncolor[[2]]}]},
     {x,Re[z0],Re[z1]},{y,Im[z0],Im[z1]},
     PlotRange->{Log[10,plotrange[[1]]],Log[10,plotrange[[2]]]},
     Ticks->{Automatic,Automatic,
     Table[{i,N[10^i]},
       {i,Floor[Log[10,plotrange[[1]]]],Floor[Log[10,plotrange[[2]]]],ticksexpstep}]},
     DisplayFunction->Identity,Evaluate[FilterOptions[Plot3D,opts]]];
   grapRatio=FullOptions[argP3,AspectRatio];
   Show[Graphics3D[argP3]/.
     Hue[h_,s_,b_]->SurfaceColor[Hue[h,s,b],RGBColor[1,1,1],specularexp],
     Lighting->True,LightSources->{{{1,0,1},Hue[0,0.5,0.6]},
       {{1,1,1},Hue[1/3,0.5,0.6]},{{0,1,1},Hue[2/3,0.5,0.6]}},
     DefaultFont->{"Courier",10*fontSizeFunct[grapRatio]},
     DisplayFunction->$DisplayFunction]]





(******************** ArgColorArcSinhPlot3D ********************)

Options[ArgColorArcSinhPlot3D]=
    Join[{AbsInterval->Automatic,
          ArgInterval->Automatic,
          IntervalColor->Automatic,
          ComplexColorSelect->Automatic,
          ComplexRegionColor->Automatic,
          SpecularExponent->Automatic,
          TicksExpStep->Automatic},Options[Plot3D]];


ArgColorArcSinhPlot3D[unknownFunc_,{z_,z0_,z1_},opts___]:=
  Module[{ukfval,x,y,argP3,grapRatio,plotrange,absinterval,arginterval,
          intervalcolor,complexcolorselect,complexregioncolor,specularexp,ticksexpstep},
    plotrange=PlotRange/.{opts}/.Options[ArgColorArcSinhPlot3D];
    absinterval=AbsInterval/.{opts}/.Options[ArgColorArcSinhPlot3D];
    arginterval=ArgInterval/.{opts}/.Options[ArgColorArcSinhPlot3D];
    intervalcolor=IntervalColor/.{opts}/.Options[ArgColorArcSinhPlot3D];
    complexcolorselect=ComplexColorSelect/.{opts}/.Options[ArgColorArcSinhPlot3D];
    complexregioncolor=ComplexRegionColor/.{opts}/.Options[ArgColorArcSinhPlot3D];
    specularexp=SpecularExponent/.{opts}/.Options[ArgColorArcSinhPlot3D];
    ticksexpstep=TicksExpStep/.{opts}/.Options[ArgColorArcSinhPlot3D];
     If[absinterval===Automatic,absinterval={1,{2,4}}];
     If[arginterval===Automatic,arginterval=8];
     If[intervalcolor===Automatic,intervalcolor={0.8,0.9}];
     If[(complexcolorselect===Automatic || complexcolorselect===PhaseAngleColor),
         complexcolorselect=1,complexcolorselect=2];
     If[complexregioncolor===Automatic,complexregioncolor={Hue[0],Hue[0.5]}];
     If[specularexp===Automatic,specularexp=200];
     If[ticksexpstep===Automatic,ticksexpstep=1];
   argP3=Plot3D[{ArcSinh[abs3DMax[ukfval=unknownFunc/.z->(x+I y),plotrange[[2]]]],
     argColorF[ukfval,complexcolorselect,
       {absinterval[[1]],{absinterval[[2,1]],absinterval[[2,2]],arginterval}},
       {intervalcolor[[1]],intervalcolor[[2]]},
       {complexregioncolor[[1]],complexregioncolor[[2]]}]},
     {x,Re[z0],Re[z1]},{y,Im[z0],Im[z1]},PlotRange->{0,ArcSinh[plotrange[[2]]]},
     Ticks->{Automatic,Automatic,
     Table[{ArcSinh[10^i],N[10^i]},{i,1,Floor[Log[10,plotrange[[2]]]],ticksexpstep}]},
     DisplayFunction->Identity,Evaluate[FilterOptions[Plot3D,opts]]];
   grapRatio=FullOptions[argP3,AspectRatio];
   Show[Graphics3D[argP3]/.
     Hue[h_,s_,b_]->SurfaceColor[Hue[h,s,b],RGBColor[1,1,1],specularexp],
     Lighting->True,LightSources->{{{1,0,1},Hue[0,0.5,0.6]},
       {{1,1,1},Hue[1/3,0.5,0.6]},{{0,1,1},Hue[2/3,0.5,0.6]}},
     DefaultFont->{"Courier",10*fontSizeFunct[grapRatio]},
     DisplayFunction->$DisplayFunction]]





(******************** RiemannSpherePlot3D ********************)

Options[RiemannSpherePlot3D]=
    Join[{RiemannSphereRadius->Automatic,
          ZeroPointAltitude->Automatic,
          FunctionMaxRange->Automatic,
          RiemannSphereType->Automatic,
          AbsInterval->Automatic,
          ArgInterval->Automatic,
          IntervalColor->Automatic,
          ComplexColorSelect->Automatic,
          ComplexRegionColor->Automatic,
          SpecularExponent->Automatic},Options[SphericalPlot3D]];


RiemannSpherePlot3D[unknownFunc_,{z_,r0_,{phi0_,phi1_}},opts___]:=
  Module[{ukfval,theta,phi,sphP3,plusZF,axes,circleFrameXZ,circleFrameYZ,
          circleFrameXY,circleFrameXY2,circleFrameXY05,circleFrameXY4,
          circleFrameXY025,point6table,grapRatio,rsphererad,zeroalt,
          funmaxrange,rspheretype,absinterval,arginterval,intervalcolor,
          complexcolorselect,complexregioncolor,specularexp},
    rsphererad=RiemannSphereRadius/.{opts}/.Options[RiemannSpherePlot3D];
    zeroalt=ZeroPointAltitude/.{opts}/.Options[RiemannSpherePlot3D];
    funmaxrange=FunctionMaxRange/.{opts}/.Options[RiemannSpherePlot3D];
    rspheretype=RiemannSphereType/.{opts}/.Options[RiemannSpherePlot3D];
    absinterval=AbsInterval/.{opts}/.Options[RiemannSpherePlot3D];
    arginterval=ArgInterval/.{opts}/.Options[RiemannSpherePlot3D];
    intervalcolor=IntervalColor/.{opts}/.Options[RiemannSpherePlot3D];
    complexcolorselect=ComplexColorSelect/.{opts}/.Options[RiemannSpherePlot3D];
    complexregioncolor=ComplexRegionColor/.{opts}/.Options[RiemannSpherePlot3D];
    specularexp=SpecularExponent/.{opts}/.Options[RiemannSpherePlot3D];
     If[rsphererad===Automatic,rsphererad=0.5];
     If[zeroalt===Automatic,zeroalt=10];
     If[funmaxrange===Automatic,funmaxrange=10];
     If[rspheretype===Automatic,rspheretype=1];
     If[absinterval===Automatic,absinterval={1,{2,4}}];
     If[arginterval===Automatic,arginterval=8];
     If[intervalcolor===Automatic,intervalcolor={0.8,0.9}];
     If[(complexcolorselect===Automatic || complexcolorselect===PhaseAngleColor),
         complexcolorselect=1,complexcolorselect=2];
     If[complexregioncolor===Automatic,complexregioncolor={Hue[0],Hue[0.5]}];
     If[specularexp===Automatic,specularexp=200];
   sphP3=SphericalPlot3D[{zeroalt+abs3DMax[
       ukfval=unknownFunc/.z->(2*rsphererad*rsTypeCot[theta/2,rspheretype]*Exp[phi I]),funmaxrange],
       {argColorF[ukfval,complexcolorselect,
       {absinterval[[1]],{absinterval[[2,1]],absinterval[[2,2]],arginterval}},
       {intervalcolor[[1]],intervalcolor[[2]]},
       {complexregioncolor[[1]],complexregioncolor[[2]]}],EdgeForm[]}},
     {theta,rsTypeThetaMin[r0,rsphererad,rspheretype],rsTypeThetaMax[r0,rsphererad,rspheretype]},
     {phi,phi0,phi1},Lighting->False,DisplayFunction->Identity,
     Evaluate[FilterOptions[SphericalPlot3D,opts]]];
   plusZF=1.4*(zeroalt+funmaxrange);
   axes=Module[{ua1},ua1=uniAng[1,rsphererad,rspheretype];
     Graphics3D[{
     Text[upplowText[1,rspheretype],{0,0,1.3*plusZF}],
        Line[{{0,0,0},{0,0,1.2*plusZF}}],
     Text[upplowText[0,rspheretype],{0,0,-1.3*plusZF}],
        Line[{{0,0,0},{0,0,-1.2*plusZF}}],
     Text["z=1",{1.3*plusZF*Sin[ua1],0,1.3*plusZF*Cos[ua1]}],
      	Line[{{0,0,0},{1.2*plusZF*Sin[ua1],0,1.2*plusZF*Cos[ua1]}}],
     Text["z=-1",{-1.3*plusZF*Sin[ua1],0,1.3*plusZF*Cos[ua1]}],
      	Line[{{0,0,0},{-1.2*plusZF*Sin[ua1],0,1.2*plusZF*Cos[ua1]}}],
     Text["z=i",{0,1.3*plusZF*Sin[ua1],1.3*plusZF*Cos[ua1]}],
      	Line[{{0,0,0},{0,1.2*plusZF*Sin[ua1],1.2*plusZF*Cos[ua1]}}],
     Text["z=-i",{0,-1.3*plusZF*Sin[ua1],1.3*plusZF*Cos[ua1]}],
      	Line[{{0,0,0},{0,-1.2*plusZF*Sin[ua1],1.2*plusZF*Cos[ua1]}}]}]];
   circleFrameXZ=
     Graphics3D[Table[{PointSize[0.002],
       Point[{plusZF*Cos[t],0,plusZF*Sin[t]}]},
       {t,0,2 Pi,Pi/300}]];
   circleFrameYZ=
     Graphics3D[Table[{PointSize[0.002],
       Point[{0,plusZF*Cos[t],plusZF*Sin[t]}]},
       {t,0,2 Pi,Pi/300}]];
   circleFrameXY=
     Module[{ua1},ua1=uniAng[1,rsphererad,rspheretype];
     Graphics3D[Table[{PointSize[0.002],
       Point[{plusZF*Cos[t]*Sin[ua1],plusZF*Sin[t]*Sin[ua1],plusZF*Cos[ua1]}]},
       {t,0,2 Pi,Pi/300}]]];
   circleFrameXY2=
     Module[{ua2},ua2=uniAng[2,rsphererad,rspheretype];
     Graphics3D[Table[{PointSize[0.0015],
       Point[{plusZF*Cos[t]*Sin[ua2],plusZF*Sin[t]*Sin[ua2],plusZF*Cos[ua2]}]},
       {t,0,2 Pi,Pi/200}]]];
   circleFrameXY05=
     Module[{ua05},ua05=uniAng[0.5,rsphererad,rspheretype];
     Graphics3D[Table[{PointSize[0.0015],
       Point[{plusZF*Cos[t]*Sin[ua05],plusZF*Sin[t]*Sin[ua05],plusZF*Cos[ua05]}]},
       {t,0,2 Pi,Pi/200}]]];
   circleFrameXY4=
     Module[{ua4},ua4=uniAng[4,rsphererad,rspheretype];
     Graphics3D[Table[{PointSize[0.001],
       Point[{plusZF*Cos[t]*Sin[ua4],plusZF*Sin[t]*Sin[ua4],plusZF*Cos[ua4]}]},
       {t,0,2 Pi,Pi/150}]]];
   circleFrameXY025=
     Module[{ua025},ua025=uniAng[0.25,rsphererad,rspheretype];
     Graphics3D[Table[{PointSize[0.001],
       Point[{plusZF*Cos[t]*Sin[ua025],plusZF*Sin[t]*Sin[ua025],plusZF*Cos[ua025]}]},
       {t,0,2 Pi,Pi/150}]]];
   point6table=
     Module[{ua1},ua1=uniAng[1,rsphererad,rspheretype];
     Graphics3D[{{PointSize[0.005],Point[{0,0,plusZF}]},
        {PointSize[0.005],Point[{0,0,-plusZF}]},
        {PointSize[0.005],Point[{plusZF*Sin[ua1],0,plusZF*Cos[ua1]}]},
        {PointSize[0.005],Point[{-plusZF*Sin[ua1],0,plusZF*Cos[ua1]}]},
        {PointSize[0.005],Point[{0,plusZF*Sin[ua1],plusZF*Cos[ua1]}]},
        {PointSize[0.005],Point[{0,-plusZF*Sin[ua1],plusZF*Cos[ua1]}]}}]];
   grapRatio=FullOptions[sphP3,AspectRatio];
   Show[{sphP3,axes,circleFrameXZ,circleFrameYZ,circleFrameXY,circleFrameXY2,
         circleFrameXY05,circleFrameXY4,circleFrameXY025,point6table}/.
     (Hue[h_,s_,b_]->SurfaceColor[Hue[h,s,b],RGBColor[1,1,1],specularexp]),
     PlotRange->All,Boxed->False,Axes->False,Lighting->True,
     LightSources->{{{1,0,1},Hue[0,0.5,0.6]},
       {{1,1,1},Hue[1/3,0.5,0.6]},{{0,1,1},Hue[2/3,0.5,0.6]}},
     DefaultFont->{"Courier",10*fontSizeFunct[grapRatio]},
     DisplayFunction->$DisplayFunction]]


rsTypeCot[z_,typ_]:=If[typ===1,Cot[z],Tan[z]];


rsTypeThetaMin[r_,rsRad_,typ_]:=If[typ===1,2*ArcCot[r/(2*rsRad)],0.00000000001];

rsTypeThetaMax[r_,rsRad_,typ_]:=If[typ===1,3.14159265358,2*ArcTan[r/(2*rsRad)]];


uniAng[r_,rsRad_,typ_]:=If[typ===1,2*ArcCot[r/(2*rsRad)],2*ArcTan[r/(2*rsRad)]];


upplowText[ul_,typ_]:=
  If[ul===1 && typ===1,"z=\[Infinity]",
  If[ul===0 && typ===1,"z=0",
  If[ul===1 && typ===2,"z=0","z=\[Infinity]"]]];





(******************** RiemannSphereDensityPlot ********************)

Options[RiemannSphereDensityPlot]=
    Join[{RiemannSphereRadius->Automatic,
          ZeroPointAltitude->Automatic,
          FunctionMaxRange->Automatic,
          RiemannSphereType->Automatic,
          AbsInterval->Automatic,
          ArgInterval->Automatic,
          IntervalColor->Automatic,
          ComplexColorSelect->Automatic,
          ComplexRegionColor->Automatic,
          SpecularExponent->Automatic},Options[SphericalPlot3D]];


RiemannSphereDensityPlot[unknownFunc_,{z_,r0_,{phi0_,phi1_}},opts___]:=
  Module[{ukfval,theta,phi,sphP3,plusZF,axes,circleFrameXZ,circleFrameYZ,
          circleFrameXY,circleFrameXY2,circleFrameXY05,circleFrameXY4,
          circleFrameXY025,point6table,grapRatio,rsphererad,zeroalt,
          funmaxrange,rspheretype,absinterval,arginterval,intervalcolor,
          complexcolorselect,complexregioncolor,specularexp},
    rsphererad=RiemannSphereRadius/.{opts}/.Options[RiemannSphereDensityPlot];
    zeroalt=ZeroPointAltitude/.{opts}/.Options[RiemannSphereDensityPlot];
    funmaxrange=FunctionMaxRange/.{opts}/.Options[RiemannSphereDensityPlot];
    rspheretype=RiemannSphereType/.{opts}/.Options[RiemannSphereDensityPlot];
    absinterval=AbsInterval/.{opts}/.Options[RiemannSphereDensityPlot];
    arginterval=ArgInterval/.{opts}/.Options[RiemannSphereDensityPlot];
    intervalcolor=IntervalColor/.{opts}/.Options[RiemannSphereDensityPlot];
    complexcolorselect=ComplexColorSelect/.{opts}/.Options[RiemannSphereDensityPlot];
    complexregioncolor=ComplexRegionColor/.{opts}/.Options[RiemannSphereDensityPlot];
    specularexp=SpecularExponent/.{opts}/.Options[RiemannSphereDensityPlot];
     If[rsphererad===Automatic,rsphererad=0.5];
     If[zeroalt===Automatic,zeroalt=10];
     If[funmaxrange===Automatic,funmaxrange=0];
     If[rspheretype===Automatic,rspheretype=1];
     If[absinterval===Automatic,absinterval={1,{2,4}}];
     If[arginterval===Automatic,arginterval=8];
     If[intervalcolor===Automatic,intervalcolor={0.8,0.9}];
     If[(complexcolorselect===Automatic || complexcolorselect===PhaseAngleColor),
         complexcolorselect=1,complexcolorselect=2];
     If[complexregioncolor===Automatic,complexregioncolor={Hue[0],Hue[0.5]}];
     If[specularexp===Automatic,specularexp=200];
   sphP3=SphericalPlot3D[{zeroalt,
       {argColorF[ukfval=unknownFunc/.z->(2*rsphererad*rsTypeCot[theta/2,rspheretype]*Exp[phi I]),
       complexcolorselect,
       {absinterval[[1]],{absinterval[[2,1]],absinterval[[2,2]],arginterval}},
       {intervalcolor[[1]],intervalcolor[[2]]},
       {complexregioncolor[[1]],complexregioncolor[[2]]}],EdgeForm[]}},
     {theta,rsTypeThetaMin[r0,rsphererad,rspheretype],rsTypeThetaMax[r0,rsphererad,rspheretype]},
     {phi,phi0,phi1},Lighting->False,DisplayFunction->Identity,
     Evaluate[FilterOptions[SphericalPlot3D,opts]]];
   plusZF=1.4*(zeroalt+funmaxrange);
   axes=Module[{ua1},ua1=uniAng[1,rsphererad,rspheretype];
     Graphics3D[{
     Text[upplowText[1,rspheretype],{0,0,1.3*plusZF}],
        Line[{{0,0,0},{0,0,1.2*plusZF}}],
     Text[upplowText[0,rspheretype],{0,0,-1.3*plusZF}],
        Line[{{0,0,0},{0,0,-1.2*plusZF}}],
     Text["z=1",{1.3*plusZF*Sin[ua1],0,1.3*plusZF*Cos[ua1]}],
      	Line[{{0,0,0},{1.2*plusZF*Sin[ua1],0,1.2*plusZF*Cos[ua1]}}],
     Text["z=-1",{-1.3*plusZF*Sin[ua1],0,1.3*plusZF*Cos[ua1]}],
      	Line[{{0,0,0},{-1.2*plusZF*Sin[ua1],0,1.2*plusZF*Cos[ua1]}}],
     Text["z=i",{0,1.3*plusZF*Sin[ua1],1.3*plusZF*Cos[ua1]}],
      	Line[{{0,0,0},{0,1.2*plusZF*Sin[ua1],1.2*plusZF*Cos[ua1]}}],
     Text["z=-i",{0,-1.3*plusZF*Sin[ua1],1.3*plusZF*Cos[ua1]}],
      	Line[{{0,0,0},{0,-1.2*plusZF*Sin[ua1],1.2*plusZF*Cos[ua1]}}]}]];
   circleFrameXZ=
     Graphics3D[Table[{PointSize[0.002],
       Point[{plusZF*Cos[t],0,plusZF*Sin[t]}]},
       {t,0,2 Pi,Pi/300}]];
   circleFrameYZ=
     Graphics3D[Table[{PointSize[0.002],
       Point[{0,plusZF*Cos[t],plusZF*Sin[t]}]},
       {t,0,2 Pi,Pi/300}]];
   circleFrameXY=
     Module[{ua1},ua1=uniAng[1,rsphererad,rspheretype];
     Graphics3D[Table[{PointSize[0.002],
       Point[{plusZF*Cos[t]*Sin[ua1],plusZF*Sin[t]*Sin[ua1],plusZF*Cos[ua1]}]},
       {t,0,2 Pi,Pi/300}]]];
   circleFrameXY2=
     Module[{ua2},ua2=uniAng[2,rsphererad,rspheretype];
     Graphics3D[Table[{PointSize[0.0015],
       Point[{plusZF*Cos[t]*Sin[ua2],plusZF*Sin[t]*Sin[ua2],plusZF*Cos[ua2]}]},
       {t,0,2 Pi,Pi/200}]]];
   circleFrameXY05=
     Module[{ua05},ua05=uniAng[0.5,rsphererad,rspheretype];
     Graphics3D[Table[{PointSize[0.0015],
       Point[{plusZF*Cos[t]*Sin[ua05],plusZF*Sin[t]*Sin[ua05],plusZF*Cos[ua05]}]},
       {t,0,2 Pi,Pi/200}]]];
   circleFrameXY4=
     Module[{ua4},ua4=uniAng[4,rsphererad,rspheretype];
     Graphics3D[Table[{PointSize[0.001],
       Point[{plusZF*Cos[t]*Sin[ua4],plusZF*Sin[t]*Sin[ua4],plusZF*Cos[ua4]}]},
       {t,0,2 Pi,Pi/150}]]];
   circleFrameXY025=
     Module[{ua025},ua025=uniAng[0.25,rsphererad,rspheretype];
     Graphics3D[Table[{PointSize[0.001],
       Point[{plusZF*Cos[t]*Sin[ua025],plusZF*Sin[t]*Sin[ua025],plusZF*Cos[ua025]}]},
       {t,0,2 Pi,Pi/150}]]];
   point6table=
     Module[{ua1},ua1=uniAng[1,rsphererad,rspheretype];
     Graphics3D[{{PointSize[0.005],Point[{0,0,plusZF}]},
        {PointSize[0.005],Point[{0,0,-plusZF}]},
        {PointSize[0.005],Point[{plusZF*Sin[ua1],0,plusZF*Cos[ua1]}]},
        {PointSize[0.005],Point[{-plusZF*Sin[ua1],0,plusZF*Cos[ua1]}]},
        {PointSize[0.005],Point[{0,plusZF*Sin[ua1],plusZF*Cos[ua1]}]},
        {PointSize[0.005],Point[{0,-plusZF*Sin[ua1],plusZF*Cos[ua1]}]}}]];
   grapRatio=FullOptions[sphP3,AspectRatio];
   Show[{sphP3,axes,circleFrameXZ,circleFrameYZ,circleFrameXY,circleFrameXY2,
         circleFrameXY05,circleFrameXY4,circleFrameXY025,point6table}/.
     (Hue[h_,s_,b_]->SurfaceColor[Hue[h,s,b],RGBColor[1,1,1],specularexp]),
     PlotRange->All,Boxed->False,Axes->False,Lighting->True,
     LightSources->{{{1,0,1},Hue[0,0.5,0.6]},
       {{1,1,1},Hue[1/3,0.5,0.6]},{{0,1,1},Hue[2/3,0.5,0.6]}},
     DefaultFont->{"Courier",10*fontSizeFunct[grapRatio]},
     DisplayFunction->$DisplayFunction]]





(******************** RiemannSphereLogPlot3D ********************)

Options[RiemannSphereLogPlot3D]=
    Join[{RiemannSphereRadius->Automatic,
          ZeroPointAltitude->Automatic,
          FunctionMaxRange->Automatic,
          RiemannSphereType->Automatic,
          AbsInterval->Automatic,
          ArgInterval->Automatic,
          IntervalColor->Automatic,
          ComplexColorSelect->Automatic,
          ComplexRegionColor->Automatic,
          SpecularExponent->Automatic},Options[SphericalPlot3D]];


RiemannSphereLogPlot3D[unknownFunc_,{z_,r0_,{phi0_,phi1_}},opts___]:=
  Module[{ukfval,theta,phi,sphP3,plusZF,axes,circleFrameXZ,circleFrameYZ,
          circleFrameXY,circleFrameXY2,circleFrameXY05,circleFrameXY4,
          circleFrameXY025,point6table,grapRatio,rsphererad,zeroalt,
          funmaxrange,rspheretype,absinterval,arginterval,intervalcolor,
          complexcolorselect,complexregioncolor,specularexp},
    rsphererad=RiemannSphereRadius/.{opts}/.Options[RiemannSphereLogPlot3D];
    zeroalt=ZeroPointAltitude/.{opts}/.Options[RiemannSphereLogPlot3D];
    funmaxrange=FunctionMaxRange/.{opts}/.Options[RiemannSphereLogPlot3D];
    rspheretype=RiemannSphereType/.{opts}/.Options[RiemannSphereLogPlot3D];
    absinterval=AbsInterval/.{opts}/.Options[RiemannSphereLogPlot3D];
    arginterval=ArgInterval/.{opts}/.Options[RiemannSphereLogPlot3D];
    intervalcolor=IntervalColor/.{opts}/.Options[RiemannSphereLogPlot3D];
    complexcolorselect=ComplexColorSelect/.{opts}/.Options[RiemannSphereLogPlot3D];
    complexregioncolor=ComplexRegionColor/.{opts}/.Options[RiemannSphereLogPlot3D];
    specularexp=SpecularExponent/.{opts}/.Options[RiemannSphereLogPlot3D];
     If[rsphererad===Automatic,rsphererad=0.5];
     If[zeroalt===Automatic,zeroalt=10];
     If[funmaxrange===Automatic,funmaxrange=10];
     If[rspheretype===Automatic,rspheretype=1];
     If[absinterval===Automatic,absinterval={1,{2,4}}];
     If[arginterval===Automatic,arginterval=8];
     If[intervalcolor===Automatic,intervalcolor={0.8,0.9}];
     If[(complexcolorselect===Automatic || complexcolorselect===PhaseAngleColor),
         complexcolorselect=1,complexcolorselect=2];
     If[complexregioncolor===Automatic,complexregioncolor={Hue[0],Hue[0.5]}];
     If[specularexp===Automatic,specularexp=200];
   sphP3=SphericalPlot3D[{zeroalt+abs3DLogMax[
       ukfval=unknownFunc/.z->(2*rsphererad*rsTypeCot[theta/2,rspheretype]*Exp[phi I]),
         {-zeroalt,funmaxrange}],
       {argColorF[ukfval,complexcolorselect,
       {absinterval[[1]],{absinterval[[2,1]],absinterval[[2,2]],arginterval}},
       {intervalcolor[[1]],intervalcolor[[2]]},
       {complexregioncolor[[1]],complexregioncolor[[2]]}],EdgeForm[]}},
     {theta,rsTypeThetaMin[r0,rsphererad,rspheretype],rsTypeThetaMax[r0,rsphererad,rspheretype]},
     {phi,phi0,phi1},Lighting->False,DisplayFunction->Identity,
     Evaluate[FilterOptions[SphericalPlot3D,opts]]];
   plusZF=1.4*(zeroalt+funmaxrange);
   axes=Module[{ua1},ua1=uniAng[1,rsphererad,rspheretype];
     Graphics3D[{
     Text[upplowText[1,rspheretype],{0,0,1.3*plusZF}],
        Line[{{0,0,0},{0,0,1.2*plusZF}}],
     Text[upplowText[0,rspheretype],{0,0,-1.3*plusZF}],
        Line[{{0,0,0},{0,0,-1.2*plusZF}}],
     Text["z=1",{1.3*plusZF*Sin[ua1],0,1.3*plusZF*Cos[ua1]}],
      	Line[{{0,0,0},{1.2*plusZF*Sin[ua1],0,1.2*plusZF*Cos[ua1]}}],
     Text["z=-1",{-1.3*plusZF*Sin[ua1],0,1.3*plusZF*Cos[ua1]}],
      	Line[{{0,0,0},{-1.2*plusZF*Sin[ua1],0,1.2*plusZF*Cos[ua1]}}],
     Text["z=i",{0,1.3*plusZF*Sin[ua1],1.3*plusZF*Cos[ua1]}],
      	Line[{{0,0,0},{0,1.2*plusZF*Sin[ua1],1.2*plusZF*Cos[ua1]}}],
     Text["z=-i",{0,-1.3*plusZF*Sin[ua1],1.3*plusZF*Cos[ua1]}],
      	Line[{{0,0,0},{0,-1.2*plusZF*Sin[ua1],1.2*plusZF*Cos[ua1]}}]}]];
   circleFrameXZ=
     Graphics3D[Table[{PointSize[0.002],
       Point[{plusZF*Cos[t],0,plusZF*Sin[t]}]},
       {t,0,2 Pi,Pi/300}]];
   circleFrameYZ=
     Graphics3D[Table[{PointSize[0.002],
       Point[{0,plusZF*Cos[t],plusZF*Sin[t]}]},
       {t,0,2 Pi,Pi/300}]];
   circleFrameXY=
     Module[{ua1},ua1=uniAng[1,rsphererad,rspheretype];
     Graphics3D[Table[{PointSize[0.002],
       Point[{plusZF*Cos[t]*Sin[ua1],plusZF*Sin[t]*Sin[ua1],plusZF*Cos[ua1]}]},
       {t,0,2 Pi,Pi/300}]]];
   circleFrameXY2=
     Module[{ua2},ua2=uniAng[2,rsphererad,rspheretype];
     Graphics3D[Table[{PointSize[0.0015],
       Point[{plusZF*Cos[t]*Sin[ua2],plusZF*Sin[t]*Sin[ua2],plusZF*Cos[ua2]}]},
       {t,0,2 Pi,Pi/200}]]];
   circleFrameXY05=
     Module[{ua05},ua05=uniAng[0.5,rsphererad,rspheretype];
     Graphics3D[Table[{PointSize[0.0015],
       Point[{plusZF*Cos[t]*Sin[ua05],plusZF*Sin[t]*Sin[ua05],plusZF*Cos[ua05]}]},
       {t,0,2 Pi,Pi/200}]]];
   circleFrameXY4=
     Module[{ua4},ua4=uniAng[4,rsphererad,rspheretype];
     Graphics3D[Table[{PointSize[0.001],
       Point[{plusZF*Cos[t]*Sin[ua4],plusZF*Sin[t]*Sin[ua4],plusZF*Cos[ua4]}]},
       {t,0,2 Pi,Pi/150}]]];
   circleFrameXY025=
     Module[{ua025},ua025=uniAng[0.25,rsphererad,rspheretype];
     Graphics3D[Table[{PointSize[0.001],
       Point[{plusZF*Cos[t]*Sin[ua025],plusZF*Sin[t]*Sin[ua025],plusZF*Cos[ua025]}]},
       {t,0,2 Pi,Pi/150}]]];
   point6table=
     Module[{ua1},ua1=uniAng[1,rsphererad,rspheretype];
     Graphics3D[{{PointSize[0.005],Point[{0,0,plusZF}]},
        {PointSize[0.005],Point[{0,0,-plusZF}]},
        {PointSize[0.005],Point[{plusZF*Sin[ua1],0,plusZF*Cos[ua1]}]},
        {PointSize[0.005],Point[{-plusZF*Sin[ua1],0,plusZF*Cos[ua1]}]},
        {PointSize[0.005],Point[{0,plusZF*Sin[ua1],plusZF*Cos[ua1]}]},
        {PointSize[0.005],Point[{0,-plusZF*Sin[ua1],plusZF*Cos[ua1]}]}}]];
   grapRatio=FullOptions[sphP3,AspectRatio];
   Show[{sphP3,axes,circleFrameXZ,circleFrameYZ,circleFrameXY,circleFrameXY2,
         circleFrameXY05,circleFrameXY4,circleFrameXY025,point6table}/.
     (Hue[h_,s_,b_]->SurfaceColor[Hue[h,s,b],RGBColor[1,1,1],specularexp]),
     PlotRange->All,Boxed->False,Axes->False,Lighting->True,
     LightSources->{{{1,0,1},Hue[0,0.5,0.6]},
       {{1,1,1},Hue[1/3,0.5,0.6]},{{0,1,1},Hue[2/3,0.5,0.6]}},
     DefaultFont->{"Courier",10*fontSizeFunct[grapRatio]},
     DisplayFunction->$DisplayFunction]]





(******************** RiemannSphereArcSinhPlot3D ********************)

Options[RiemannSphereArcSinhPlot3D]=
    Join[{RiemannSphereRadius->Automatic,
          ZeroPointAltitude->Automatic,
          FunctionMaxRange->Automatic,
          RiemannSphereType->Automatic,
          AbsInterval->Automatic,
          ArgInterval->Automatic,
          IntervalColor->Automatic,
          ComplexColorSelect->Automatic,
          ComplexRegionColor->Automatic,
          SpecularExponent->Automatic},Options[SphericalPlot3D]];


RiemannSphereArcSinhPlot3D[unknownFunc_,{z_,r0_,{phi0_,phi1_}},opts___]:=
  Module[{ukfval,theta,phi,sphP3,plusZF,axes,circleFrameXZ,circleFrameYZ,
          circleFrameXY,circleFrameXY2,circleFrameXY05,circleFrameXY4,
          circleFrameXY025,point6table,grapRatio,rsphererad,zeroalt,
          funmaxrange,rspheretype,absinterval,arginterval,intervalcolor,
          complexcolorselect,complexregioncolor,specularexp},
    rsphererad=RiemannSphereRadius/.{opts}/.Options[RiemannSphereArcSinhPlot3D];
    zeroalt=ZeroPointAltitude/.{opts}/.Options[RiemannSphereArcSinhPlot3D];
    funmaxrange=FunctionMaxRange/.{opts}/.Options[RiemannSphereArcSinhPlot3D];
    rspheretype=RiemannSphereType/.{opts}/.Options[RiemannSphereArcSinhPlot3D];
    absinterval=AbsInterval/.{opts}/.Options[RiemannSphereArcSinhPlot3D];
    arginterval=ArgInterval/.{opts}/.Options[RiemannSphereArcSinhPlot3D];
    intervalcolor=IntervalColor/.{opts}/.Options[RiemannSphereArcSinhPlot3D];
    complexcolorselect=ComplexColorSelect/.{opts}/.Options[RiemannSphereArcSinhPlot3D];
    complexregioncolor=ComplexRegionColor/.{opts}/.Options[RiemannSphereArcSinhPlot3D];
    specularexp=SpecularExponent/.{opts}/.Options[RiemannSphereArcSinhPlot3D];
     If[rsphererad===Automatic,rsphererad=0.5];
     If[zeroalt===Automatic,zeroalt=10];
     If[funmaxrange===Automatic,funmaxrange=10];
     If[rspheretype===Automatic,rspheretype=1];
     If[absinterval===Automatic,absinterval={1,{2,4}}];
     If[arginterval===Automatic,arginterval=8];
     If[intervalcolor===Automatic,intervalcolor={0.8,0.9}];
     If[(complexcolorselect===Automatic || complexcolorselect===PhaseAngleColor),
         complexcolorselect=1,complexcolorselect=2];
     If[complexregioncolor===Automatic,complexregioncolor={Hue[0],Hue[0.5]}];
     If[specularexp===Automatic,specularexp=200];
   sphP3=SphericalPlot3D[{zeroalt+abs3DArcSinhMax[
       ukfval=unknownFunc/.z->(2*rsphererad*rsTypeCot[theta/2,rspheretype]*Exp[phi I]),funmaxrange],
       {argColorF[ukfval,complexcolorselect,
       {absinterval[[1]],{absinterval[[2,1]],absinterval[[2,2]],arginterval}},
       {intervalcolor[[1]],intervalcolor[[2]]},
       {complexregioncolor[[1]],complexregioncolor[[2]]}],EdgeForm[]}},
     {theta,rsTypeThetaMin[r0,rsphererad,rspheretype],rsTypeThetaMax[r0,rsphererad,rspheretype]},
     {phi,phi0,phi1},Lighting->False,DisplayFunction->Identity,
     Evaluate[FilterOptions[SphericalPlot3D,opts]]];
   plusZF=1.4*(zeroalt+funmaxrange);
   axes=Module[{ua1},ua1=uniAng[1,rsphererad,rspheretype];
     Graphics3D[{
     Text[upplowText[1,rspheretype],{0,0,1.3*plusZF}],
        Line[{{0,0,0},{0,0,1.2*plusZF}}],
     Text[upplowText[0,rspheretype],{0,0,-1.3*plusZF}],
        Line[{{0,0,0},{0,0,-1.2*plusZF}}],
     Text["z=1",{1.3*plusZF*Sin[ua1],0,1.3*plusZF*Cos[ua1]}],
      	Line[{{0,0,0},{1.2*plusZF*Sin[ua1],0,1.2*plusZF*Cos[ua1]}}],
     Text["z=-1",{-1.3*plusZF*Sin[ua1],0,1.3*plusZF*Cos[ua1]}],
      	Line[{{0,0,0},{-1.2*plusZF*Sin[ua1],0,1.2*plusZF*Cos[ua1]}}],
     Text["z=i",{0,1.3*plusZF*Sin[ua1],1.3*plusZF*Cos[ua1]}],
      	Line[{{0,0,0},{0,1.2*plusZF*Sin[ua1],1.2*plusZF*Cos[ua1]}}],
     Text["z=-i",{0,-1.3*plusZF*Sin[ua1],1.3*plusZF*Cos[ua1]}],
      	Line[{{0,0,0},{0,-1.2*plusZF*Sin[ua1],1.2*plusZF*Cos[ua1]}}]}]];
   circleFrameXZ=
     Graphics3D[Table[{PointSize[0.002],
       Point[{plusZF*Cos[t],0,plusZF*Sin[t]}]},
       {t,0,2 Pi,Pi/300}]];
   circleFrameYZ=
     Graphics3D[Table[{PointSize[0.002],
       Point[{0,plusZF*Cos[t],plusZF*Sin[t]}]},
       {t,0,2 Pi,Pi/300}]];
   circleFrameXY=
     Module[{ua1},ua1=uniAng[1,rsphererad,rspheretype];
     Graphics3D[Table[{PointSize[0.002],
       Point[{plusZF*Cos[t]*Sin[ua1],plusZF*Sin[t]*Sin[ua1],plusZF*Cos[ua1]}]},
       {t,0,2 Pi,Pi/300}]]];
   circleFrameXY2=
     Module[{ua2},ua2=uniAng[2,rsphererad,rspheretype];
     Graphics3D[Table[{PointSize[0.0015],
       Point[{plusZF*Cos[t]*Sin[ua2],plusZF*Sin[t]*Sin[ua2],plusZF*Cos[ua2]}]},
       {t,0,2 Pi,Pi/200}]]];
   circleFrameXY05=
     Module[{ua05},ua05=uniAng[0.5,rsphererad,rspheretype];
     Graphics3D[Table[{PointSize[0.0015],
       Point[{plusZF*Cos[t]*Sin[ua05],plusZF*Sin[t]*Sin[ua05],plusZF*Cos[ua05]}]},
       {t,0,2 Pi,Pi/200}]]];
   circleFrameXY4=
     Module[{ua4},ua4=uniAng[4,rsphererad,rspheretype];
     Graphics3D[Table[{PointSize[0.001],
       Point[{plusZF*Cos[t]*Sin[ua4],plusZF*Sin[t]*Sin[ua4],plusZF*Cos[ua4]}]},
       {t,0,2 Pi,Pi/150}]]];
   circleFrameXY025=
     Module[{ua025},ua025=uniAng[0.25,rsphererad,rspheretype];
     Graphics3D[Table[{PointSize[0.001],
       Point[{plusZF*Cos[t]*Sin[ua025],plusZF*Sin[t]*Sin[ua025],plusZF*Cos[ua025]}]},
       {t,0,2 Pi,Pi/150}]]];
   point6table=
     Module[{ua1},ua1=uniAng[1,rsphererad,rspheretype];
     Graphics3D[{{PointSize[0.005],Point[{0,0,plusZF}]},
        {PointSize[0.005],Point[{0,0,-plusZF}]},
        {PointSize[0.005],Point[{plusZF*Sin[ua1],0,plusZF*Cos[ua1]}]},
        {PointSize[0.005],Point[{-plusZF*Sin[ua1],0,plusZF*Cos[ua1]}]},
        {PointSize[0.005],Point[{0,plusZF*Sin[ua1],plusZF*Cos[ua1]}]},
        {PointSize[0.005],Point[{0,-plusZF*Sin[ua1],plusZF*Cos[ua1]}]}}]];
   grapRatio=FullOptions[sphP3,AspectRatio];
   Show[{sphP3,axes,circleFrameXZ,circleFrameYZ,circleFrameXY,circleFrameXY2,
         circleFrameXY05,circleFrameXY4,circleFrameXY025,point6table}/.
     (Hue[h_,s_,b_]->SurfaceColor[Hue[h,s,b],RGBColor[1,1,1],specularexp]),
     PlotRange->All,Boxed->False,Axes->False,Lighting->True,
     LightSources->{{{1,0,1},Hue[0,0.5,0.6]},
       {{1,1,1},Hue[1/3,0.5,0.6]},{{0,1,1},Hue[2/3,0.5,0.6]}},
     DefaultFont->{"Courier",10*fontSizeFunct[grapRatio]},
     DisplayFunction->$DisplayFunction]]





(******************** ReAndImPlot3D ********************)

Options[ReAndImPlot3D]=
    Join[{PlotArray->Automatic,
          ReColorPair->Automatic,
          ImColorPair->Automatic,
          MaxColor->Automatic,
          ColorSeparateNum->Automatic,
          SpecularExponent->Automatic},Options[Plot3D]];


ReAndImPlot3D[unknownFunc_,{z_,z0_,z1_},opts___]:=
  Module[{ukfval,x,y,reP3,imP3,reshow,imshow,grapRatio,plotrange,plotarray,
      recolorpair,imcolorpair,maxcolor,colorsepnum,specularexp},
    plotrange=PlotRange/.{opts}/.Options[ReAndImPlot3D];
    plotarray=PlotArray/.{opts}/.Options[ReAndImPlot3D];
    recolorpair=ReColorPair/.{opts}/.Options[ReAndImPlot3D];
    imcolorpair=ImColorPair/.{opts}/.Options[ReAndImPlot3D];
    maxcolor=MaxColor/.{opts}/.Options[ReAndImPlot3D];
    colorsepnum=ColorSeparateNum/.{opts}/.Options[ReAndImPlot3D];
    specularexp=SpecularExponent/.{opts}/.Options[ReAndImPlot3D];
      If[(plotarray===Automatic || plotarray===Abscissa),
        plotarray={reshow,imshow},plotarray={{reshow},{imshow}}];
      If[recolorpair===Automatic,recolorpair={Hue[0.5],Hue[0]}];
      If[imcolorpair===Automatic,imcolorpair={Hue[0.75],Hue[0.25]}];
      If[maxcolor===Automatic,
        maxcolor={{GrayLevel[0.3],GrayLevel[0.3]},{GrayLevel[0.3],GrayLevel[0.3]}}];
      If[colorsepnum===Automatic,colorsepnum=8];
      If[specularexp===Automatic,specularexp=30];
    clipfillRe[f_]:=
      If[Re[f]<=plotrange[[1]],
        plotrange[[1]]+0.00001*(plotrange[[2]]-plotrange[[1]]),
      If[Re[f]>=plotrange[[2]],
        plotrange[[2]]-0.00001*(plotrange[[2]]-plotrange[[1]]),Re[f]]];
    clipfillIm[f_]:=
      If[Im[f]<=plotrange[[1]],
        plotrange[[1]]+0.00001*(plotrange[[2]]-plotrange[[1]]),
      If[Im[f]>=plotrange[[2]],
        plotrange[[2]]-0.00001*(plotrange[[2]]-plotrange[[1]]),Im[f]]];
    reP3=Plot3D[{clipfillRe[ukfval=unknownFunc/.z->(x+I y)],
      reimColorScale[clipfillRe[ukfval],plotRangeAbsMax[plotrange],colorsepnum,
        {recolorpair[[1]],recolorpair[[2]]},{maxcolor[[1,1]],maxcolor[[1,2]]}]},
      {x,Re[z0],Re[z1]},{y,Im[z0],Im[z1]},DisplayFunction->Identity,
      Evaluate[FilterOptions[Plot3D,opts]]];
    grapRatio=FullOptions[reP3,AspectRatio];
    reshow=Show[Graphics3D[reP3]/.
      Hue[h_,s_,b_]->SurfaceColor[Hue[h,s,b],RGBColor[1,1,1],specularexp],
      Lighting->True,LightSources->{{{1,0,1},Hue[0,0.5,0.6]},
        {{1,1,1},Hue[1/3,0.5,0.6]},{{0,1,1},Hue[2/3,0.5,0.6]}},
      DefaultFont->{"Courier",10*fontSizeFunct[grapRatio]},
      DisplayFunction->Identity];
    imP3=Plot3D[{clipfillIm[ukfval=unknownFunc/.z->(x+I y)],
      reimColorScale[clipfillIm[ukfval],plotRangeAbsMax[plotrange],colorsepnum,
        {imcolorpair[[1]],imcolorpair[[2]]},{maxcolor[[2,1]],maxcolor[[2,2]]}]},
      {x,Re[z0],Re[z1]},{y,Im[z0],Im[z1]},DisplayFunction->Identity,
      Evaluate[FilterOptions[Plot3D,opts]]];
    imshow=Show[Graphics3D[imP3]/.
      Hue[h_,s_,b_]->SurfaceColor[Hue[h,s,b],RGBColor[1,1,1],specularexp],
      Lighting->True,LightSources->{{{1,0,1},Hue[0,0.5,0.6]},
        {{1,1,1},Hue[1/3,0.5,0.6]},{{0,1,1},Hue[2/3,0.5,0.6]}},
      DefaultFont->{"Courier",10*fontSizeFunct[grapRatio]},
      DisplayFunction->Identity];
    Show[GraphicsArray[plotarray,GraphicsSpacing->0.02]]]


reimColorScale[h_,aprMax_,m_,{cNega_,cPosi_},{cNegaMax_,cPosiMax_}]:=
  If[h<0,reimCSFunc[-h,aprMax,m,{cNega,cNegaMax}],
         reimCSFunc[h,aprMax,m,{cPosi,cPosiMax}]];

reimCSFunc[h_,aprMax_,m_,{cInit_,cBlen_}]:=
  Module[{ci,cb},
    ci=changeHue[cBlen[[1]],cInit];
    cb=changeHue[cInit[[1]],cBlen];
   Apply[Hue,Map[((1-ricsf[h,aprMax,m])*ci[[#]]+ricsf[h,aprMax,m]*cb[[#]])&,{1,2,3}]]];

ricsf[h_,aprMax_,m_]:=Floor[m*sawWave[h/aprMax]]/(2 m);


plotRangeAbsMax[prList_]:=Max[zrAbs[prList[[1]]],zrAbs[prList[[2]]]];





(******************** RePlot3D ********************)

Options[RePlot3D]=
    Join[{ReColorPair->Automatic,
          MaxColor->Automatic,
          ColorSeparateNum->Automatic,
          SpecularExponent->Automatic},Options[Plot3D]];


RePlot3D[unknownFunc_,{z_,z0_,z1_},opts___]:=
  Module[{ukfval,x,y,reP3,grapRatio,plotrange,recolorpair,maxcolor,
          colorsepnum,specularexp},
    plotrange=PlotRange/.{opts}/.Options[RePlot3D];
    recolorpair=ReColorPair/.{opts}/.Options[RePlot3D];
    maxcolor=MaxColor/.{opts}/.Options[RePlot3D];
    colorsepnum=ColorSeparateNum/.{opts}/.Options[RePlot3D];
    specularexp=SpecularExponent/.{opts}/.Options[RePlot3D];
      If[recolorpair===Automatic,recolorpair={Hue[0.5],Hue[0]}];
      If[maxcolor===Automatic,maxcolor={GrayLevel[0.3],GrayLevel[0.3]}];
      If[colorsepnum===Automatic,colorsepnum=8];
      If[specularexp===Automatic,specularexp=30];
    clipfillRe[f_]:=
      If[Re[f]<=plotrange[[1]],
        plotrange[[1]]+0.00001*(plotrange[[2]]-plotrange[[1]]),
      If[Re[f]>=plotrange[[2]],
        plotrange[[2]]-0.00001*(plotrange[[2]]-plotrange[[1]]),Re[f]]];
    reP3=Plot3D[{clipfillRe[ukfval=unknownFunc/.z->(x+I y)],
      reimColorScale[clipfillRe[ukfval],plotRangeAbsMax[plotrange],colorsepnum,
        {recolorpair[[1]],recolorpair[[2]]},{maxcolor[[1]],maxcolor[[2]]}]},
      {x,Re[z0],Re[z1]},{y,Im[z0],Im[z1]},DisplayFunction->Identity,
      Evaluate[FilterOptions[Plot3D,opts]]];
    grapRatio=FullOptions[reP3,AspectRatio];
    Show[Graphics3D[reP3]/.
      Hue[h_,s_,b_]->SurfaceColor[Hue[h,s,b],RGBColor[1,1,1],specularexp],
      Lighting->True,LightSources->{{{1,0,1},Hue[0,0.5,0.6]},
        {{1,1,1},Hue[1/3,0.5,0.6]},{{0,1,1},Hue[2/3,0.5,0.6]}},
      DefaultFont->{"Courier",10*fontSizeFunct[grapRatio]},
      DisplayFunction->$DisplayFunction]]





(******************** ImPlot3D ********************)

Options[ImPlot3D]=
    Join[{ImColorPair->Automatic,
          MaxColor->Automatic,
          ColorSeparateNum->Automatic,
          SpecularExponent->Automatic},Options[Plot3D]];


ImPlot3D[unknownFunc_,{z_,z0_,z1_},opts___]:=
  Module[{ukfval,x,y,imP3,grapRatio,plotrange,imcolorpair,maxcolor,
          colorsepnum,specularexp},
    plotrange=PlotRange/.{opts}/.Options[ImPlot3D];
    imcolorpair=ImColorPair/.{opts}/.Options[ImPlot3D];
    maxcolor=MaxColor/.{opts}/.Options[ImPlot3D];
    colorsepnum=ColorSeparateNum/.{opts}/.Options[ImPlot3D];
    specularexp=SpecularExponent/.{opts}/.Options[ImPlot3D];
      If[imcolorpair===Automatic,imcolorpair={Hue[0.75],Hue[0.25]}];
      If[maxcolor===Automatic,maxcolor={GrayLevel[0.3],GrayLevel[0.3]}];
      If[colorsepnum===Automatic,colorsepnum=8];
      If[specularexp===Automatic,specularexp=30];
    clipfillIm[f_]:=
      If[Im[f]<=plotrange[[1]],
        plotrange[[1]]+0.00001*(plotrange[[2]]-plotrange[[1]]),
      If[Im[f]>=plotrange[[2]],
        plotrange[[2]]-0.00001*(plotrange[[2]]-plotrange[[1]]),Im[f]]];
    imP3=Plot3D[{clipfillIm[ukfval=unknownFunc/.z->(x+I y)],
      reimColorScale[clipfillIm[ukfval],plotRangeAbsMax[plotrange],colorsepnum,
        {imcolorpair[[1]],imcolorpair[[2]]},{maxcolor[[1]],maxcolor[[2]]}]},
      {x,Re[z0],Re[z1]},{y,Im[z0],Im[z1]},DisplayFunction->Identity,
      Evaluate[FilterOptions[Plot3D,opts]]];
    grapRatio=FullOptions[imP3,AspectRatio];
    Show[Graphics3D[imP3]/.
      Hue[h_,s_,b_]->SurfaceColor[Hue[h,s,b],RGBColor[1,1,1],specularexp],
      Lighting->True,LightSources->{{{1,0,1},Hue[0,0.5,0.6]},
        {{1,1,1},Hue[1/3,0.5,0.6]},{{0,1,1},Hue[2/3,0.5,0.6]}},
      DefaultFont->{"Courier",10*fontSizeFunct[grapRatio]},
      DisplayFunction->$DisplayFunction]]





(******************** AbsAndArgDensityPlot ********************)

Options[AbsAndArgDensityPlot]=
  Join[{PlotArray->Automatic,
        AbsoluteMaxRange->Automatic},Options[DensityPlot]];


AbsAndArgDensityPlot[unknownFunc_,{z_,z0_,z1_},opts___]:=
  Module[{x,y,abspl,argpl,absplShow,argplShow,grapRatio,
          absolutemaxrange,plotarray},
   absolutemaxrange=AbsoluteMaxRange/.{opts}/.Options[AbsAndArgDensityPlot];
   plotarray=PlotArray/.{opts}/.Options[AbsAndArgDensityPlot];
    If[absolutemaxrange===Automatic,absolutemaxrange=100000000];
    If[(plotarray===Automatic || plotarray===Abscissa),
      plotarray={absplShow,argplShow},plotarray={{absplShow},{argplShow}}];
  abspl=DensityPlot[zrAbs[unknownFunc/.z->(x+I y)],
    {x,Re[z0],Re[z1]},{y,Im[z0],Im[z1]},
    PlotRange->{0,absolutemaxrange},
    PlotLabel->"Abs",ColorFunction->(Hue[Log[10,10 #],1,1]&),
    DisplayFunction->Identity,Evaluate[FilterOptions[DensityPlot,opts]]];
  grapRatio=FullOptions[abspl,AspectRatio];
  absplShow=Show[abspl,DefaultFont->{"Courier",10*fontSizeFunct[grapRatio]},
    DisplayFunction->Identity];
  argpl=DensityPlot[zrArg[unknownFunc/.z->(x+I y)],
    {x,Re[z0],Re[z1]},{y,Im[z0],Im[z1]},PlotRange->{-Pi,Pi},
    PlotLabel->"Arg",ColorFunction->(Hue[#+0.5,1,1]&),
    DisplayFunction->Identity,Evaluate[FilterOptions[DensityPlot,opts]]];
  argplShow=Show[argpl,DefaultFont->{"Courier",10*fontSizeFunct[grapRatio]},
    DisplayFunction->Identity];
  Show[GraphicsArray[plotarray,GraphicsSpacing->0.01]]]





(******************** FundamentalRegionPlot ********************)

Options[FundamentalRegionPlot]=
  Join[{FundamentalRegionColor->Automatic,
        FundamentalRegionType->Automatic,
        AbsContourRange->Automatic,
        AbsContour->Automatic,
        ArgContour->Automatic,
        AbsContourStyle->Automatic,
        ArgContourStyle->Automatic},
    Intersection[Options[DensityPlot],Options[ContourPlot]]];


FundamentalRegionPlot[unknownFunc_,{z_,z0_,z1_},opts___]:=
  Module[{x,y,regionDensP,absContP,argContP,grapRatio,fRegionColor,fRegionType,
          absContRange,absContour,argContour,absContStyle,argContStyle},
   fRegionColor=FundamentalRegionColor/.{opts}/.Options[FundamentalRegionPlot];
   fRegionType=FundamentalRegionType/.{opts}/.Options[FundamentalRegionPlot];
   absContRange=AbsContourRange/.{opts}/.Options[FundamentalRegionPlot];
   absContour=AbsContour/.{opts}/.Options[FundamentalRegionPlot];
   argContour=ArgContour/.{opts}/.Options[FundamentalRegionPlot];
   absContStyle=AbsContourStyle/.{opts}/.Options[FundamentalRegionPlot];
   argContStyle=ArgContourStyle/.{opts}/.Options[FundamentalRegionPlot];
    If[fRegionColor===Automatic,fRegionColor={Hue[0],Hue[0.5]}];
    If[fRegionType===Automatic,fRegionType=2];
    If[absContRange===Automatic,absContRange={10^-30,10^30}];
    If[absContour===Automatic,absContour={1,{2,4}}];
    If[argContour===Automatic,argContour=8];
    If[absContStyle===Automatic,
       absContStyle={{GrayLevel[0],Thickness[0.0005]}}];
    If[argContStyle===Automatic,
       argContStyle={{Hue[0.6+(fRegionType-1)/10],Thickness[0.0005]}}];
 regionDensP=
   DensityPlot[zrArgPi[unknownFunc/.z->N[x+I y]],
   {x,Re[z0],Re[z1]},{y,Im[z0],Im[z1]},PlotRange->{-Pi,Pi},Mesh->False,
   ColorFunction->(densPColorFunc[#,{fRegionType,{fRegionColor[[1]],fRegionColor[[2]]}}]&),
   DisplayFunction->Identity,Evaluate[FilterOptions[DensityPlot,opts]]];
 absContP=
   ContourPlot[zrAbs[unknownFunc/.z->(x+I y)],
   {x,Re[z0],Re[z1]},{y,Im[z0],Im[z1]},ContourShading->False,
   PlotRange->{0,absContRange[[2]]},
   Contours->Table[contourGapFunc[{absContour[[1]],{absContour[[2,1]],absContour[[2,2]]}},m],
     {m,Floor[contMinMax[{absContour[[1]],{absContour[[2,1]],absContour[[2,2]]}},absContRange[[1]]]],
     Ceiling[contMinMax[{absContour[[1]],{absContour[[2,1]],absContour[[2,2]]}},absContRange[[2]]]],1}],
   ContourStyle->absContStyle,DisplayFunction->Identity,
   Evaluate[FilterOptions[ContourPlot,opts]]];
 argContP=
   ContourPlot[absOfArg[unknownFunc/.z->(x+I y)],
   {x,Re[z0],Re[z1]},{y,Im[z0],Im[z1]},ContourShading->False,
   PlotRange->{0,Pi},Contours->Table[m Pi/(2*argContour),{m,1,2*argContour-1,1}],
   ContourStyle->argContStyle,DisplayFunction->Identity,
   Evaluate[FilterOptions[ContourPlot,opts]]];
 grapRatio=FullOptions[regionDensP,AspectRatio];
 Show[{regionDensP,argContP,absContP},PlotRange->All,
    DefaultFont->{"Courier",10*fontSizeFunct[grapRatio]},
    DisplayFunction->$DisplayFunction]]


zrArgPi[z_]:=If[NumericQ[z]=!=True,-Pi,If[Abs[z]>0,Arg[z],-Pi]];


contourGapFunc[{ftype_,{baseN_,absN_}},z_]:=
  If[ftype===1,baseN^(z/absN),
  If[ftype===2,If[z<0,baseN^(z/absN),1+Log[baseN]*(z/absN)],
  If[ftype===3,(baseN^(baseN^(z/absN))-1)/(baseN-1),
  If[ftype===4,(baseN^(baseN^(baseN^(z/absN)))-baseN)/(baseN^baseN-baseN),
  If[ftype===5,
    If[z<0,(baseN-1)/(baseN^(baseN^(-z/absN))-1),
           (baseN^(baseN^(z/absN))-1)/(baseN-1)],
    If[z<0,(baseN^baseN-baseN)/(baseN^(baseN^(baseN^(-z/absN)))-baseN),
           (baseN^(baseN^(baseN^(z/absN)))-baseN)/(baseN^baseN-baseN)]]]]]];

contMinMax[{ftype_,{baseN_,absN_}},z_]:=
  If[ftype===1,absN*Log[baseN,z],
  If[ftype===2,absN*If[z<1,Log[baseN,z],(z-1)/Log[baseN]],
  If[ftype===3,absN*Log[baseN,Log[baseN,(baseN-1)*z+1]],
  If[ftype===4,absN*Log[baseN,Log[baseN,Log[baseN,(baseN^baseN-baseN)*z+baseN]]],
  If[ftype===5,
    If[z<1,-absN*Log[baseN,Log[baseN,(baseN-1)/z+1]],
           absN*Log[baseN,Log[baseN,(baseN-1)*z+1]]],
    If[z<1,-absN*Log[baseN,Log[baseN,Log[baseN,(baseN^baseN-baseN)/z+baseN]]],
           absN*Log[baseN,Log[baseN,Log[baseN,(baseN^baseN-baseN)*z+baseN]]]]]]]]];


densPColorFunc[z_,{type_,{posiColor_,negaColor_}}]:=
  If[type===1,If[0<z<1/2,negaColor,
               If[z>=1/2,posiColor,
                         whiteFunc[posiColor]]],
              If[0<z<1/2,Head[negaColor][z/5+Part[negaColor,1]],
               If[z>=1/2,Head[posiColor][z/5+Part[posiColor,1]-0.1],
                         whiteFunc[posiColor]]]];

whiteFunc[col_]:=If[Head[col]===Hue,Hue[0,0,1],GrayLevel[1]]





(******************** FundamentalRegionPlot2 ********************)

Options[FundamentalRegionPlot2]=
  Join[{FundamentalRegionColor->Automatic,
        FundamentalRegionType->Automatic,
        AbsContourRange->Automatic,
        AbsContour->Automatic,
        ArgContour->Automatic,
        AbsContourStyle->Automatic,
        ArgContourStyle->Automatic},
    Intersection[Options[DensityPlot],Options[ContourPlot]]];


FundamentalRegionPlot2[unknownFunc_,{z_,z0_,z1_},opts___]:=
  Module[{x,y,regionDensP,absContP,argContP,grapRatio,fRegionColor,fRegionType,
          absContRange,absContour,argContour,absContStyle,argContStyle},
   fRegionColor=FundamentalRegionColor/.{opts}/.Options[FundamentalRegionPlot2];
   fRegionType=FundamentalRegionType/.{opts}/.Options[FundamentalRegionPlot2];
   absContRange=AbsContourRange/.{opts}/.Options[FundamentalRegionPlot2];
   absContour=AbsContour/.{opts}/.Options[FundamentalRegionPlot2];
   argContour=ArgContour/.{opts}/.Options[FundamentalRegionPlot2];
   absContStyle=AbsContourStyle/.{opts}/.Options[FundamentalRegionPlot2];
   argContStyle=ArgContourStyle/.{opts}/.Options[FundamentalRegionPlot2];
    If[fRegionColor===Automatic,fRegionColor={Hue[0],Hue[0.5]}];
    If[fRegionType===Automatic,fRegionType=2];
    If[absContRange===Automatic,absContRange={10^-30,10^30}];
    If[absContour===Automatic,absContour={1,{2,4}}];
    If[argContour===Automatic,argContour=8];
    If[absContStyle===Automatic,
       absContStyle={{GrayLevel[0],Thickness[0.0005]}}];
    If[argContStyle===Automatic,
       argContStyle={{Hue[0.6+(fRegionType-1)/10],Thickness[0.0005]}}];
 regionDensP=
   DensityPlot[zrArgPi[unknownFunc/.z->(x+I y)],
   {x,Re[z0],Re[z1]},{y,Im[z0],Im[z1]},PlotRange->{-Pi,Pi},Mesh->False,
   ColorFunction->(densPColorFunc[#,{fRegionType,{fRegionColor[[1]],fRegionColor[[2]]}}]&),
   DisplayFunction->Identity,Evaluate[FilterOptions[DensityPlot,opts]]];
 absContP=
   ContourPlot[Log[zrAbs[unknownFunc/.z->(x+I y)]],
   {x,Re[z0],Re[z1]},{y,Im[z0],Im[z1]},ContourShading->False,
   PlotRange->{0,Log[absContRange[[2]]]},
   Contours->Table[Log[contourGapFunc[{absContour[[1]],{absContour[[2,1]],absContour[[2,2]]}},m]],
     {m,Floor[contMinMax[{absContour[[1]],{absContour[[2,1]],absContour[[2,2]]}},absContRange[[1]]]],
     Ceiling[contMinMax[{absContour[[1]],{absContour[[2,1]],absContour[[2,2]]}},absContRange[[2]]]],1}],
   ContourStyle->absContStyle,DisplayFunction->Identity,
   Evaluate[FilterOptions[ContourPlot,opts]]];
 argContP=
   ContourPlot[absOfArg[unknownFunc/.z->(x+I y)],
   {x,Re[z0],Re[z1]},{y,Im[z0],Im[z1]},ContourShading->False,
   PlotRange->{0,Pi},Contours->Table[m Pi/(2*argContour),{m,1,2*argContour-1,1}],
   ContourStyle->argContStyle,DisplayFunction->Identity,
   Evaluate[FilterOptions[ContourPlot,opts]]];
 grapRatio=FullOptions[regionDensP,AspectRatio];
 Show[{regionDensP,argContP,absContP},PlotRange->All,
    DefaultFont->{"Courier",10*fontSizeFunct[grapRatio]},
    DisplayFunction->$DisplayFunction]]





(******************** DensityContourPlot ********************)

Options[DensityContourPlot]=
  Join[{DensityRegionColor->Automatic,
        AbsContourRange->Automatic,
        AbsContour->Automatic,
        ArgContour->Automatic,
        AbsContourStyle->Automatic,
        ArgContourStyle->Automatic},
    Intersection[Options[DensityPlot],Options[ContourPlot]]];


DensityContourPlot[unknownFunc_,{z_,z0_,z1_},opts___]:=
  Module[{x,y,densContDensP,absContP,argContP1,argContP2,grapRatio,densContPColor,
          absContRange,absContour,argContour,absContStyle,argContStyle},
   densContPColor=DensityRegionColor/.{opts}/.Options[DensityContourPlot];
   absContRange=AbsContourRange/.{opts}/.Options[DensityContourPlot];
   absContour=AbsContour/.{opts}/.Options[DensityContourPlot];
   argContour=ArgContour/.{opts}/.Options[DensityContourPlot];
   absContStyle=AbsContourStyle/.{opts}/.Options[DensityContourPlot];
   argContStyle=ArgContourStyle/.{opts}/.Options[DensityContourPlot];
    If[absContRange===Automatic,absContRange={10^-30,10^30}];
    If[absContour===Automatic,absContour={1,{2,4}}];
    If[argContour===Automatic,argContour=8];
    If[densContPColor===Automatic,densContPColor=argContour];
    If[absContStyle===Automatic,absContStyle={{GrayLevel[0],Thickness[0.0005]}}];
    If[argContStyle===Automatic,
      argContStyle={{AbsoluteDashing[{3,3}],GrayLevel[0],Thickness[0.0005]}}];
 densContDensP=
   DensityPlot[zrArg[unknownFunc/.z->(x+I y)],
   {x,Re[z0],Re[z1]},{y,Im[z0],Im[z1]},PlotRange->{-Pi,Pi},Mesh->False,
   ColorFunction->(dcPRainbowColorFunc[#,densContPColor]&),
   DisplayFunction->Identity,Evaluate[FilterOptions[DensityPlot,opts]]];
 absContP=
   ContourPlot[zrAbs[unknownFunc/.z->(x+I y)],
   {x,Re[z0],Re[z1]},{y,Im[z0],Im[z1]},ContourShading->False,
   PlotRange->{0,absContRange[[2]]},
   Contours->Table[contourGapFunc[{absContour[[1]],{absContour[[2,1]],absContour[[2,2]]}},m],
     {m,Floor[contMinMax[{absContour[[1]],{absContour[[2,1]],absContour[[2,2]]}},absContRange[[1]]]],
     Ceiling[contMinMax[{absContour[[1]],{absContour[[2,1]],absContour[[2,2]]}},absContRange[[2]]]],1}],
   ContourStyle->absContStyle,DisplayFunction->Identity,
   Evaluate[FilterOptions[ContourPlot,opts]]];
 argContP1=
   ContourPlot[absOfArg[unknownFunc/.z->(x+I y)],
   {x,Re[z0],Re[z1]},{y,Im[z0],Im[z1]},ContourShading->False,
   Contours->Table[m Pi/(2*argContour),{m,1,2*argContour-1,1}],
   ContourStyle->argContStyle,DisplayFunction->Identity,
   Evaluate[FilterOptions[ContourPlot,opts]]];
 argContP2=
   ContourPlot[Im[unknownFunc/.z->(x+I y)],
   {x,Re[z0],Re[z1]},{y,Im[z0],Im[z1]},ContourShading->False,Contours->{0},
   ContourStyle->argContStyle,DisplayFunction->Identity,
   Evaluate[FilterOptions[ContourPlot,opts]]];
 grapRatio=FullOptions[densContDensP,AspectRatio];
 Show[{densContDensP,argContP1,argContP2,absContP},PlotRange->All,
    DefaultFont->{"Courier",10*fontSizeFunct[grapRatio]},
    DisplayFunction->$DisplayFunction]]


dcPRainbowColorFunc[z_,colorNumb_]:=
  If[colorNumb===Infinity,Hue[z+0.5],Hue[Floor[4 colorNumb*z]/(4 colorNumb)+0.5]];





(******************** RegionAbsAndArgDensityPlot ********************)

Options[RegionAbsAndArgDensityPlot]=
  Join[{PlotArray->Automatic,
        AbsInterval->Automatic,
        RegionOutsideColor->Automatic,
        ComplexColorSelect->Automatic,
        ComplexRegionColor->Automatic},Options[DensityPlot]];


RegionAbsAndArgDensityPlot[unknownFunc_,unknownRegion_,{z_,z0_,z1_},opts___]:=
  Module[{x,y,abspl,argpl,absplShow,argplShow,grapRatio,plotarray,absinterval,
          regionoutsidecolor,complexcolorselect,complexregioncolor},
   plotarray=PlotArray/.{opts}/.Options[RegionAbsAndArgDensityPlot];
   absinterval=AbsInterval/.{opts}/.Options[RegionAbsAndArgDensityPlot];
   regionoutsidecolor=RegionOutsideColor/.{opts}/.Options[RegionAbsAndArgDensityPlot];
   complexcolorselect=ComplexColorSelect/.{opts}/.Options[RegionAbsAndArgDensityPlot];
   complexregioncolor=ComplexRegionColor/.{opts}/.Options[RegionAbsAndArgDensityPlot];
    If[(plotarray===Automatic || plotarray===Abscissa),
      plotarray={absplShow,argplShow},plotarray={{absplShow},{argplShow}}];
    If[absinterval===Automatic,absinterval={1,{2,1}}];
    If[regionoutsidecolor===Automatic,regionoutsidecolor={1,0.6}];
    If[(complexcolorselect===Automatic || complexcolorselect===PhaseAngleColor),
        complexcolorselect=1,complexcolorselect=2];
    If[complexregioncolor===Automatic,complexregioncolor={Hue[0],Hue[0.5]}];
  abspl=DensityPlot[
    regAbsValueFunct[unknownFunc/.z->(x+I y),unknownRegion/.z->(x+I y),
      {absinterval[[1]],{absinterval[[2,1]],absinterval[[2,2]]}}],
    {x,Re[z0],Re[z1]},{y,Im[z0],Im[z1]},PlotRange->{0,2},
    ColorFunction->(regAbsColorF[#,complexcolorselect,
      {regionoutsidecolor[[1]],regionoutsidecolor[[2]]},
      {complexregioncolor[[1]],complexregioncolor[[2]]}]&),
    DisplayFunction->Identity,Evaluate[FilterOptions[DensityPlot,opts]]];
  grapRatio=FullOptions[abspl,AspectRatio];
  absplShow=Show[abspl,DefaultFont->{"Courier",10*fontSizeFunct[grapRatio]},
    DisplayFunction->Identity];
  argpl=DensityPlot[
    regArgValueFunct[unknownFunc/.z->(x+I y),unknownRegion/.z->(x+I y)],
    {x,Re[z0],Re[z1]},{y,Im[z0],Im[z1]},PlotRange->{0,4 Pi},
    ColorFunction->(regArgColorF[#,complexcolorselect,
      {regionoutsidecolor[[1]],regionoutsidecolor[[2]]},
      {complexregioncolor[[1]],complexregioncolor[[2]]}]&),
    DisplayFunction->Identity,Evaluate[FilterOptions[DensityPlot,opts]]];
  argplShow=Show[argpl,DefaultFont->{"Courier",10*fontSizeFunct[grapRatio]},
    DisplayFunction->Identity];
  Show[GraphicsArray[plotarray,GraphicsSpacing->0.01]]]


regAbsValueFunct[z_,r_,{intFType_,{logIntV_,absIntV_}}]:=
  If[r===True,sawWave[absIntervalFunc[intFType,logIntV,z]*absIntV],
              sawWave[absIntervalFunc[intFType,logIntV,z]*absIntV]+1];

regAbsColorF[h_,ccSel_,{iSatu_,iBrig_},{c1_,c2_}]:=
  If[h<=1/2,ccSHue[2 h,1,1,ccSel,{c1,c2}],
            ccSHue[2 h,iSatu,iBrig,ccSel,{c1,c2}]];


regArgValueFunct[z_,r_]:=If[r===True,zrArg[z]+Pi,zrArg[z]+3*Pi];

regArgColorF[h_,ccSel_,{iSatu_,iBrig_},{c1_,c2_}]:=
  If[h<=1/2,ccSHue[2 h+0.5,1,1,ccSel,{c1,c2}],
            ccSHue[2 h+0.5,iSatu,iBrig,ccSel,{c1,c2}]];





(******************** RegionAbsDensityPlot ********************)

Options[RegionAbsDensityPlot]=
    Join[{AbsInterval->Automatic,
          RegionOutsideColor->Automatic,
          ComplexColorSelect->Automatic,
          ComplexRegionColor->Automatic},Options[DensityPlot]];


RegionAbsDensityPlot[unknownFunc_,unknownRegion_,{z_,z0_,z1_},opts___]:=
  Module[{x,y,abspl,grapRatio,absinterval,regionoutsidecolor,
          complexcolorselect,complexregioncolor},
    absinterval=AbsInterval/.{opts}/.Options[RegionAbsDensityPlot];
    regionoutsidecolor=RegionOutsideColor/.{opts}/.Options[RegionAbsDensityPlot];
    complexcolorselect=ComplexColorSelect/.{opts}/.Options[RegionAbsDensityPlot];
    complexregioncolor=ComplexRegionColor/.{opts}/.Options[RegionAbsDensityPlot];
     If[absinterval===Automatic,absinterval={1,{2,1}}];
     If[regionoutsidecolor===Automatic,regionoutsidecolor={1,0.6}];
     If[(complexcolorselect===Automatic || complexcolorselect===PhaseAngleColor),
         complexcolorselect=1,complexcolorselect=2];
     If[complexregioncolor===Automatic,complexregioncolor={Hue[0],Hue[0.5]}];
   abspl=DensityPlot[
     regAbsValueFunct[unknownFunc/.z->(x+I y),unknownRegion/.z->(x+I y),
       {absinterval[[1]],{absinterval[[2,1]],absinterval[[2,2]]}}],
     {x,Re[z0],Re[z1]},{y,Im[z0],Im[z1]},PlotRange->{0,2},
     ColorFunction->(regAbsColorF[#,complexcolorselect,
       {regionoutsidecolor[[1]],regionoutsidecolor[[2]]},
       {complexregioncolor[[1]],complexregioncolor[[2]]}]&),
     DisplayFunction->Identity,Evaluate[FilterOptions[DensityPlot,opts]]];
   grapRatio=FullOptions[abspl,AspectRatio];
   Show[abspl,DefaultFont->{"Courier",10*fontSizeFunct[grapRatio]},
     DisplayFunction->$DisplayFunction]]





(******************** RegionArgDensityPlot ********************)

Options[RegionArgDensityPlot]=
    Join[{RegionOutsideColor->Automatic,
        ComplexColorSelect->Automatic,
        ComplexRegionColor->Automatic},Options[DensityPlot]];


RegionArgDensityPlot[unknownFunc_,unknownRegion_,{z_,z0_,z1_},opts___]:=
  Module[{x,y,argpl,grapRatio,regionoutsidecolor,complexcolorselect,
          complexregioncolor},
    regionoutsidecolor=RegionOutsideColor/.{opts}/.Options[RegionArgDensityPlot];
    complexcolorselect=ComplexColorSelect/.{opts}/.Options[RegionArgDensityPlot];
    complexregioncolor=ComplexRegionColor/.{opts}/.Options[RegionArgDensityPlot];
     If[regionoutsidecolor===Automatic,regionoutsidecolor={1,0.6}];
     If[(complexcolorselect===Automatic || complexcolorselect===PhaseAngleColor),
         complexcolorselect=1,complexcolorselect=2];
     If[complexregioncolor===Automatic,complexregioncolor={Hue[0],Hue[0.5]}];
   argpl=DensityPlot[
     regArgValueFunct[unknownFunc/.z->(x+I y),unknownRegion/.z->(x+I y)],
     {x,Re[z0],Re[z1]},{y,Im[z0],Im[z1]},PlotRange->{0,4 Pi},
     ColorFunction->(regArgColorF[#,complexcolorselect,
       {regionoutsidecolor[[1]],regionoutsidecolor[[2]]},
       {complexregioncolor[[1]],complexregioncolor[[2]]}]&),
     DisplayFunction->Identity,Evaluate[FilterOptions[DensityPlot,opts]]];
   grapRatio=FullOptions[argpl,AspectRatio];
   Show[argpl,DefaultFont->{"Courier",10*fontSizeFunct[grapRatio]},
     DisplayFunction->$DisplayFunction]]





(******************** End ********************)

End[]

Protect[
  PoleRemovalPlot, GridLinesPlot, CommonGridLinesPlot, 
  GridLinesParametricPlot, GridLinesColor, FundamentalRectangle, 
  AbsColorPlot3D, AbsColorDensityPlot, ArgColorPlot3D, ArgColorDensityPlot, 
  ArgColorLogPlot3D, ArgColorArcSinhPlot3D, 
  AbsInterval, ArgInterval, IntervalColor, ComplexColorSelect, PhaseAngleColor, 
  PhaseRegionColor, ComplexRegionColor, SpecularExponent, TicksExpStep, 
  RiemannSpherePlot3D, RiemannSphereDensityPlot, RiemannSphereLogPlot3D, 
  RiemannSphereArcSinhPlot3D, 
  RiemannSphereRadius, ZeroPointAltitude, FunctionMaxRange, RiemannSphereType, 
  ReAndImPlot3D, RePlot3D, ImPlot3D, ReColorPair, ImColorPair, MaxColor, 
  ColorSeparateNum, PlotArray, Abscissa, Ordinate, 
  AbsAndArgDensityPlot, AbsoluteMaxRange, 
  FundamentalRegionPlot, FundamentalRegionPlot2, FundamentalRegionColor, 
  FundamentalRegionType, AbsContourRange, AbsContour, ArgContour, 
  AbsContourStyle, ArgContourStyle, 
  DensityContourPlot, DensityRegionColor, 
  RegionAbsAndArgDensityPlot, RegionAbsDensityPlot, RegionArgDensityPlot, 
  RegionOutsideColor]

EndPackage[]